<?php

namespace SyncSpider\APIextension\Api\Data;

interface OrderInterface extends \Magento\Framework\Api\ExtensibleDataInterface
{

    /**
     * Gets the increment ID for the order
     *
     * @api
     *
     * @return string|null
     */
    public function getIncrementId();

    /**
     * Sets the increment ID for the order
     *
     * @api
     *
     * @param string $incrementId
     *
     * @return void
     */
    public function setIncrementId($incrementId);

    /**
     * Gets the external order ID for the order.
     *
     * @api
     *
     * @return string|null External order ID.
     */
    public function getExtOrderId();

    /**
     * Sets the external order ID for the order.
     *
     * @param string $id
     *
     * @return $this
     */
    public function setExtOrderId($id);

    /**
     * Gets currency code for the order
     *
     * @api
     *
     * @return string
     */
    public function getCurrencyCode();

    /**
     * Sets currency code for the order
     *
     * @api
     *
     * @param string $currencyCode
     *
     * @return void
     */
    public function setCurrencyCode($currencyCode);

    /**
     * Gets the status for the order
     *
     * @api
     *
     * @return string
     */
    public function getStatus();

    /**
     * Sets the status for the order
     *
     * @api
     *
     * @param string $status
     *
     * @return void
     */
    public function setStatus($status);

    /**
     * Gets products for the order
     *
     * @api
     *
     * @return \SyncSpider\APIextension\Api\Data\OrderItemInterface[]
     */
    public function getProducts();

    /**
     * Sets products for the order
     *
     * @api
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderItemInterface $products
     *
     * @return void
     */
    public function setProducts($products);

    /**
     * Gets customer for the order
     *
     * @api
     *
     * @return \SyncSpider\APIextension\Api\Data\OrderCustomerInterface
     */
    public function getCustomer();

    /**
     * Sets customer for the order
     *
     * @api
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderCustomerInterface $customer
     *
     * @return void
     */
    public function setCustomer($customer);

    /**
     * Gets the billing address for the order
     *
     * @api
     *
     * @return \SyncSpider\APIextension\Api\Data\OrderCustomerAddressInterface
     */
    public function getBillingAddress();

    /**
     * Sets the billing address for the order
     *
     * @api
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderCustomerAddressInterface $billingAddress
     *
     * @return void
     */
    public function setBillingAddress($billingAddress);

    /**
     * Gets the shipping address for the order
     *
     * @api
     *
     * @return \SyncSpider\APIextension\Api\Data\OrderCustomerAddressInterface
     */
    public function getShippingAddress();

    /**
     * Sets the shipping address for the order
     *
     * @api
     * @param \SyncSpider\APIextension\Api\Data\OrderCustomerAddressInterface $shippingAddress
     *
     * @return void
     */
    public function setShippingAddress($shippingAddress);

    /**
     * Gets order payment
     *
     * @api
     *
     * @return \SyncSpider\APIextension\Api\Data\OrderPaymentInterface
     */
    public function getPayment();

    /**
     * Sets order payment
     *
     * @api
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderPaymentInterface $payment
     *
     * @return void
     */
    public function setPayment($payment);

    /**
     * Gets order shipping
     *
     * @api
     *
     * @return \SyncSpider\APIextension\Api\Data\OrderShippingInterface
     */
    public function getShipping();

    /**
     * Sets order shipping
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderShippingInterface $shipping
     *
     * @return void
     */
    public function setShipping($shipping);

    /**
     * Gets shipping tax class id
     *
     * @api
     *
     * @return int
     */
    public function getShippingTaxClassId();

    /**
     * Sets shipping tax class id
     *
     * @api
     *
     * @param int $shippingTaxClassId
     *
     * @return void
     */
    public function setShippingTaxClassId($shippingTaxClassId);

    /**
     * Gets order comment
     *
     * @api
     *
     * @return string
     */
    public function getComment();

    /**
     * Sets order comment
     *
     * @api
     *
     * @param string $comment
     *
     * @return void
     */
    public function setComment($comment);

    /**
     * Gets the coupon code for the order
     *
     * @api
     *
     * @return string
     */
    public function getCouponCode();

    /**
     * Sets the coupon code for the order
     *
     * @api
     *
     * @param string $couponCode
     *
     * @return void
     */
    public function setCouponCode($couponCode);

    /**
     * Whether prices include tax
     *
     * @api
     *
     * @return bool
     */
    public function isPriceIncludeTax();

    /**
     * Sets whether prices include tax
     *
     * @api
     *
     * @param bool $priceIncludeTax
     *
     * @return void
     */
    public function setPriceIncludeTax($priceIncludeTax);

    /**
     * Whether create customer associated with the order
     *
     * @api
     *
     * @return bool
     */
    public function isCreateCustomer();

    /**
     * Sets whether create customer associated with the order
     *
     * @api
     *
     * @param bool $createCustomer
     *
     * @return void
     */
    public function setCreateCustomer($createCustomer);

    /**
     * Whether shipping address is same as billing address
     *
     * @api
     *
     * @return bool
     */
    public function isSameAsBilling();

    /**
     * Sets whether shipping address is same as billing address
     *
     * @api
     * @param bool $sameAsBilling
     * @return void
     */
    public function setSameAsBilling($sameAsBilling);

    /**
     * Whether create products associated with the order
     *
     * @api
     *
     * @return bool
     */
    public function isCreateProducts();

    /**
     * Sets whether create products associated with the order
     *
     * @api
     *
     * @param bool $createProducts
     *
     * @return void
     */
    public function setCreateProducts($createProducts);

    /**
     * Gets shipping method for the order
     *
     * @return string
     */
    public function getShippingMethod();

    /**
     * Gets payment method for the order
     *
     * @return string
     */
    public function getPaymentMethod();

    /**
     * Gets the customer email address for the order
     *
     * @return string
     */
    public function getCustomerEmail();

    /**
     * Whether customer associated with the order is an guest
     *
     * @return bool
     */
    public function isGuestCustomer();

    /**
     * Gets customer group id
     *
     * @return string
     */
    public function getCustomerGroupId();

    /**
     * Whether creation of customer associated with the order is possible
     *
     * @return bool
     */
    public function shouldCreateCustomer();

    /**
     * Whether creation of products associated with the order is possible
     *
     * @return bool
     */
    public function shouldCreateProduct();

    /**
     * Gets account associated with the order
     *
     * @return mixed
     */
    public function getAccount();

    /**
     * Sets account associated with the order
     *
     * @param mixed $account
     *
     * @return void
     */
    public function setAccount($account);

    /**
     * Convert order data to array implementation
     *
     * @param array $keys
     *
     * @return array
     */
    public function toArray(array $keys = []);

    /**
     * Retrieve existing extension attributes object or create a new one.
     *
     * @return \Magento\Sales\Api\Data\OrderExtensionInterface|null
     */
    public function getExtensionAttributes();

    /**
     * Set an extension attributes object.
     *
     * @param \Magento\Sales\Api\Data\OrderExtensionInterface $extensionAttributes
     * @return $this
     */
    public function setExtensionAttributes(\Magento\Sales\Api\Data\OrderExtensionInterface $extensionAttributes);
}
