<?php

namespace SyncSpider\APIextension\Api\Data;

interface OrderItemInterface
{
    /**
     * Gets the SKU for the order item
     *
     * @api
     *
     * @return string
     */
    public function getSku();

    /**
     * Sets the SKU for the order item.
     *
     * @api
     *
     * @param string $sku
     */
    public function setSku($sku);

    /**
     * Gets the name for the order item
     *
     * @api
     *
     * @return string
     */
    public function getName();

    /**
     * Sets the name for the order item
     *
     * @api
     *
     * @param string $name
     */
    public function setName($name);

    /**
     * Gets the attribute set id for the order item
     *
     * @api
     *
     * @return string
     */
    public function getAttributeSetId();

    /**
     * Sets the attribute set id for the order item
     *
     * @api
     *
     * @param string $attributeSetId
     */
    public function setAttributeSetId($attributeSetId);

    /**
     * Gets the custom price for the order item
     *
     * @api
     *
     * @return float
     */
    public function getCustomPrice();

    /**
     * Sets the custom price for the order item
     *
     * @api
     *
     * @param float $customPrice
     */
    public function setCustomPrice($customPrice);

    /**
     * Gets the invoiced qty for the order item
     *
     * @api
     *
     * @return string
     */
    public function getQty();

    /**
     * Sets the invoiced qty for the order item
     *
     * @api
     *
     * @param string $qty
     */
    public function setQty($qty);

    /**
     * Gets the ordered qty for the order item
     *
     * @api
     *
     * @return string
     */
    public function getOrderedQty();

    /**
     * Sets the ordered qty for the order item
     *
     * @api
     *
     * @param string $orderedQty
     */
    public function setOrderedQty($orderedQty);

    /**
     * Get gift message associated with the order item
     *
     * @api
     *
     * @return \SyncSpider\APIextension\Api\Data\OrderItemGiftMessageInterface
     */
    public function getGiftMessage();

    /**
     * Set gift message associated with the order item
     *
     * @api
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderItemGiftMessageInterface $giftMessage
     */
    public function setGiftMessage($giftMessage);

    /**
     * Get tax class id associated with the order item
     *
     * @api
     *
     * @return string
     */
    public function getTaxClassId();

    /**
     * Set tax class id associated with the order item
     *
     * @api
     *
     * @param string $taxClassId
     */
    public function setTaxClassId($taxClassId);

    /**
     * Wether discount is associated with the order item
     *
     * @api
     *
     * @return bool
     */
    public function isUseDiscount();

    /**
     * Set wether associate discount with the order item
     *
     * @api
     *
     * @param bool $useDiscount
     */
    public function setUseDiscount($useDiscount);

    /**
     * Returns product option
     *
     * @return \Magento\Catalog\Api\Data\ProductOptionInterface|null
     */
    public function getProductOption();

    /**
     * Sets product option
     *
     * @param \Magento\Catalog\Api\Data\ProductOptionInterface $productOption
     * @return $this
     */
    public function setProductOption(\Magento\Catalog\Api\Data\ProductOptionInterface $productOption);
}
