<?php

namespace SyncSpider\APIextension\Api\Data;

interface OrderPaymentInterface
{
    /**
     * Get payment method code
     *
     * @api
     *
     * @return string
     */
    public function getPaymentMethod();

    /**
     * Set payment method code
     *
     * @api
     *
     * @param string $paymentMethod
     */
    public function setPaymentMethod($paymentMethod);

    /**
     * Get external order id associated with the payment
     *
     * @api
     *
     * @return string
     */
    public function getExternalOrderId();

    /**
     * Set external order id associated with the payment
     *
     * @api
     *
     * @param string $externalOrderId
     */
    public function setExternalOrderId($externalOrderId);

    /**
     * Get external tax id associated with the payment
     *
     * @api
     *
     * @return string
     */
    public function getExternalTaxId();

    /**
     * Set external tax id associated with the payment
     *
     * @api
     *
     * @param string $externalTaxId
     */
    public function setExternalTaxId($externalTaxId);

    /**
     * Get external transaction id associated with the payment
     *
     * @api
     *
     * @return string
     */
    public function getExternalTransactionId();

    /**
     * Set external transaction id associated with the payment
     *
     * @api
     *
     * @param string $externalTransactionId
     */
    public function setExternalTransactionId($externalTransactionId);

    /**
     * Get external transaction date associated with the payment
     *
     * @api
     *
     * @return string
     */
    public function getExternalTransactionDate();

    /**
     * Set external transaction date associated with the payment
     *
     * @api
     *
     * @param string $externalTransactionDate
     */
    public function setExternalTransactionDate($externalTransactionDate);

    /**
     * Get external order (transaction) fee associated with the payment
     *
     * @api
     *
     * @return string
     */
    public function getExternalOrderFee();

    /**
     * Set external order (transaction) fee associated with the payment
     *
     * @api
     *
     * @param string $externalOrderFee
     */
    public function setExternalOrderFee($externalOrderFee);

    /**
     * Get payment additional information
     *
     * @api
     *
     * @return \SyncSpider\APIextension\Api\Data\OrderPaymentAdditionalInterface[]
     */
    public function getPaymentAdditional();

    /**
     * Set payment additional information
     *
     * @api
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderPaymentAdditionalInterface[] $paymentAdditional
     */
    public function setPaymentAdditional(array $paymentAdditional);

    /**
     * Get external payment system name
     *
     * @api
     *
     * @return string
     */
    public function getExternalSystemName();

    /**
     * Set external payment system name
     *
     * @api
     *
     * @param string $externalSystemName
     */
    public function setExternalSystemName($externalSystemName);
}
