<?php

namespace SyncSpider\APIextension\Api\Data;

use Magento\Catalog\Api\Data\ProductInterface as MagentoProductInterface;

interface ProductInterface extends MagentoProductInterface
{
    /**
     * Retrieve parent sku collection
     *
     * @return \SyncSpider\APIextension\Api\Data\ProductParentInterface[]|null
     */
    public function getParents();

    /**
     * Retrieve Product URL
     *
     * @return string
     */
    public function getVisibleProductUrl();

    /**
     * Get store codes where product is available
     *
     * @return string[]
     */
    public function getStoreCodes();

    /**
     * Get website ids the product is assigned to
     *
     * @return int[]
     */
    public function getWebsiteIds();

    /**
     * Get product pirce including tax
     *
     * @return float
     */
    public function getPriceInclTax();

    /**
     * Get product price excluding tax
     *
     * @return float
     */
    public function getPriceExclTax();

    /**
     * Get product final price including tax
     *
     * @return float
     */
    public function getFinalPriceInclTax();

    /**
     * Get product final price excluding tax
     *
     * @return float
     */
    public function getFinalPriceExclTax();

    /**
     * Get categoriy ids the product is assigned to
     *
     * @return int[]
     */
    public function getCategories();

    /**
     * Get product gallery images
     *
     * @return \Magento\Catalog\Api\Data\ProductAttributeMediaGalleryEntryInterface[]|null
     */
    public function getImages();
}
