<?php

namespace SyncSpider\APIextension\Api;

use DomainException;

interface EntityTypeRepositoryInterface
{
    /**
     * Get entity type data based on the given entity type
     *
     * @param string|null $entityType Valid entity_type_code, entity data interface class name or entity_type_id
     *
     * @throws DomainException
     *
     * @return array
     */
    public function getEntityTypeData($entityType): array;

    /**
     * Get entity type id based on the given entity type
     *
     * @param string|null $entityType Valid entity_type_code, entity data interface class name or entity_type_id
     *
     * @throws DomainException
     *
     * @return integer
     */
    public function getEntityTypeId($entityType): int;

    /**
     * Get entity type code based on the given entity type
     *
     * @param string|null $entityType Valid entity_type_code, entity data interface class name or entity_type_id
     *
     * @throws DomainException
     *
     * @return integer
     */
    public function getEntityTypeCode($entityType): string;

    /**
     * Get entity type interface name for given entity type
     *
     * @param string|int $entityType Valid entity_type_code, entity data interface class name or entity_type_id
     *
     * @return string|null
     */
    public function getEntityType($entityType): ?string;
}
