<?php

namespace SyncSpider\APIextension\Api\ExtensionAttributes;

use Magento\Framework\Api\ExtensibleDataInterface;
use Magento\Framework\Api\ExtensionAttributesInterface;

interface ProcessorContextInterface
{
    /**
     * Set processor context
     *
     * @param array $context
     *
     * @return \SyncSpider\APIextension\Api\ExtensionAttributes\ProcessorContextInterface
     */
    public function setContext(array $context): ProcessorContextInterface;

    /**
     * Get processor context
     *
     * @return array
     */
    public function retrieveContext(): array;

    /**
     * Whether given entity is valid for processor
     *
     * @return boolean
     */
    public function validEntity($entity): bool;

    /**
     * Get extenion attributes for given entity
     *
     * @param \Magento\Framework\Api\ExtensionAttributesInterface $entity
     *
     * @return \Magento\Framework\Api\ExtensionAttributesInterface
     */
    public function getEntityExtensionAttributes(ExtensibleDataInterface $entity): ExtensionAttributesInterface;

    /**
     * Set processor context value
     *
     * @param string $context
     * @param mixed $value
     *
     * @return \SyncSpider\APIextension\Api\ExtensionAttributes\ProcessorContextInterface
     */
    public function setContextValue(string $context, $value): ProcessorContextInterface;

    /**
     * Get processor context value
     *
     * @param string $context
     *
     * @return mixed
     */
    public function getContextValue(string $context);
}
