<?php

namespace SyncSpider\APIextension\Api;

interface InventoryStockFeatureInterface
{
    /**
     * Enable inventory stock feature funcionallity for all stock related funcionality
     *
     * @return InventoryStockFeatureInterface
     */
    public function enable(): InventoryStockFeatureInterface;

    /**
     * Disable previously enabled inventory stock feature for all stock related funcionality
     *
     * @return InventoryStockFeatureInterface
     */
    public function disable(): InventoryStockFeatureInterface;

    /**
     * Whether inventory stock feature is enabled for all stock related funcionality
     *
     * @return boolean
     */
    public function active(): bool;

    /**
     * Enable inventory stock feature for all stock related funcionality only for given callback
     *
     * In case the feature has been enabled already the feature won't be disabled after executing callback
     * In case the feature is disabled the feature will be enabled before executing callback and disabled after
     * executing the callback
     *
     * @param callable $callback Function to execute
     * @param array $arguments List of arguments to pass to given callback
     *
     * @return mixed Return value of executed callback
     */
    public function execute(callable $callback, array $arguments = []);
}
