<?php
namespace SyncSpider\APIextension\Block\Adminhtml\Webhook\Edit\Tab;

class WebhookGrid extends \Magento\Backend\Block\Widget\Form\Generic implements
    \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * Prepare form
     *
     * @return $this
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function _prepareForm()
    {
        /** @var $model \Magento\Cms\Model\Page */
        $model = $this->_coreRegistry->registry('apiextension_webhook');

        $isElementDisabled = false;

        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create();

        $form->setHtmlIdPrefix('page_');

        $fieldset = $form->addFieldset('base_fieldset', ['legend' => __('Webhook Options')]);

        if ($model->getId()) {
            $fieldset->addField('id', 'hidden', ['name' => 'id']);
        }

        $optionArray = [ 0 => __('No'), 1 => __('Yes') ];

        $fieldset->addField(
            'event_code',
            'text',
            [
                'name' => 'event_code',
                'label' => __('Event code'),
                'title' => __('Event code'),
                'required' => true,
            ]
        );
        $fieldset->addField(
            'description',
            'textarea',
            [
                'name' => 'description',
                'label' => __('Description'),
                'title' => __('Description'),
            ]
        );
        $fieldset->addField(
            'token',
            'text',
            [
                'name' => 'token',
                'label' => __('Token'),
                'title' => __('Token'),
                'required' => true,
            ]
        );
        $fieldset->addField(
            'callback_url',
            'text',
            [
                'name' => 'callback_url',
                'label' => __('Callback URL'),
                'title' => __('Callback URL'),
                'required' => true,
            ]
        );
        $fieldset->addField(
            'data',
            'text',
            [
                'name' => 'data',
                'label' => __('Data'),
                'title' => __('Data'),
            ]
        );
        $fieldset->addField(
            'active',
            'select',
            [
                'name' => 'active',
                'label' => __('Active'),
                'title' => __('Active'),
                'values' => $optionArray,
                'value' => $model->getData('active'),
                'required' => true,
            ]
        );
        /*{{CedAddFormField}}*/

        if (!$model->getId()) {
            $model->setData('status', $isElementDisabled ? '2' : '1');
        }

        $form->setValues($model->getData());
        $this->setForm($form);

        return parent::_prepareForm();
    }

    /**
     * Prepare label for tab
     *
     * @return string
     */
    public function getTabLabel()
    {
        return __('Webhook data');
    }

    /**
     * Prepare title for tab
     *
     * @return string
     */
    public function getTabTitle()
    {
        return __('Webhook data');
    }

    /**
     * @inheritdoc
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * @inheritdoc
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * Check permission for passed action
     *
     * @param  string $resourceId
     * @return bool
     */
    protected function _isAllowedAction($resourceId)
    {
        return $this->_authorization->isAllowed($resourceId);
    }
}
