<?php

namespace SyncSpider\APIextension\EntityManager;

use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;

class GenericCollectionHandler implements ExtensionInterface
{
    /**
     * @var ExtensionAttributesResolver
     */
    private $extensionAttributesResolver;

    /**
     * @var string
     */
    private $actionName;

    public function __construct(
        ExtensionAttributesResolver $extensionAttributesResolver,
        string $actionName = null
    ) {
        $this->extensionAttributesResolver = $extensionAttributesResolver;
        $this->actionName = $actionName;
    }

    /**
     * Execute specific entity action on every collection item
     *
     * Make sure that these actions work on direct data accessible within
     * the loaded entity otherwise create custom collection handler which
     * loads necessary data for all collection entities  
     *
     * @param AbstractDb $entity
     * @param array $arguments
     *
     * @return void
     */
    public function execute($entity, $arguments = [])
    {
        if ($entity instanceof AbstractDb
            && ! $this->extensionAttributesResolver->hasActionFlag($entity, $this->actionName)
        ) {
            $this->extensionAttributesResolver->setActionFlag($entity, $this->actionName);
            $entityType = $arguments['entityType'] ?? null;

            if ($entityType && interface_exists($entityType)) {
                foreach ($entity->getItems() as $item) {
                    $this->extensionAttributesResolver->triggerEntityActions($item, $entityType, $this->actionName);
                }
            }
        }
    }
}
