<?php

namespace SyncSpider\APIextension\EntityManager;

use Magento\Framework\{
    Api\CustomAttributesDataInterface,
    Api\AbstractSimpleObject,
    Api\SimpleDataObjectConverter,
    DataObject
};
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Sales\Api\Data\OrderAddressInterface;
use SyncSpider\APIextension\EntityManager\OrderAddressSaveHandler;

class OrderAddressReadHandler implements ExtensionInterface
{
    /**
     * @var OrderAddressSaveHandler
     */
    private $saveHandler;

    public function __construct(OrderAddressSaveHandler $saveHandler)
    {
        $this->saveHandler = $saveHandler;
    }

    /**
     * Process sales order address entity
     * 
     * Utilizes the save hanler "map" to map order custom attribute values to
     * its extension attribute value. Only scalar custom attribute values are
     * assigned to extension attributes.
     *
     * @param OrderAddressInterface $entity
     * @param array $arguments
     *
     * @return object|bool
     */
    public function execute($entity, $arguments = [])
    {
        $map = $this->saveHandler->getMap();

        if ($entity instanceof OrderAddressInterface
            && $entity instanceof CustomAttributesDataInterface
        ) {
            $extensionAttributes = $entity->getExtensionAttributes();

            if ($extensionAttributes instanceof AbstractSimpleObject) {
                foreach ($map as $extensionAttribute => $attributeCode) {
                    $setter = SimpleDataObjectConverter::snakeCaseToCamelCase('set_' . $extensionAttribute);

                    if (method_exists($extensionAttributes, $setter)) {
                        $customAttribute = $entity->getCustomAttribute($attributeCode);
                        $value = null;
    
                        if ($customAttribute) {
                            $value = $customAttribute->getValue();
                        } elseif ($entity instanceof DataObject) {
                            // fallback to set data in case custom attribute functionality does not work
                            $value = $entity->getData($attributeCode);
                        }
    
                        if (is_scalar($value)) {
                            $extensionAttributes->{$setter}($value);
                        }
                    }
                }
            }
        }

        return $entity;
    }
}
