<?php

namespace SyncSpider\APIextension\EntityManager;

use Magento\Framework\{
    Api\CustomAttributesDataInterface,
    Api\AbstractSimpleObject,
    DataObject
};
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Sales\Api\Data\OrderAddressInterface;

class OrderAddressSaveHandler implements ExtensionInterface
{
    /**
     * @var string[]
     */
    private $map;

    public function __construct(array $map = [])
    {
        $this->map = $map;
    }

    /**
     * Map of extension attribute to custom attribute code
     *
     * @return array
     */
    public function getMap(): array
    {
        return $this->map;
    }

    /**
     * Process sales order address entity
     * 
     * Utilizes the "map" property to map order entity custo attribute values to
     * its extension attribute value. Only scalar extension attribute values are
     * assigned to custom attributes
     *
     * @param OrderAddressInterface $entity
     * @param array $arguments
     *
     * @return object|bool
     */
    public function execute($entity, $arguments = [])
    {
        if ($entity instanceof OrderAddressInterface
            && $entity instanceof CustomAttributesDataInterface
        ) {
            $extensionAttributes = $entity->getExtensionAttributes();

            if ($extensionAttributes instanceof AbstractSimpleObject) {
                $extensionAttributes = $extensionAttributes->__toArray();

                foreach ($extensionAttributes as $extensionAttribute => $value) {
                    if (! array_key_exists($extensionAttribute, $this->map)) {
                        continue;
                    }

                    if (is_scalar($value)) {
                        $entity->setCustomAttribute($extensionAttribute, $value);
                        $customAttribute = $entity->getCustomAttribute($extensionAttribute);

                        // fallback to set data in case custom attribute functionality does not work
                        if (! $customAttribute && $entity instanceof DataObject) {
                            $entity->setData($extensionAttribute, $value);
                        }
                    }
                }
            }
        }

        return $entity;
    }
}
