<?php

namespace SyncSpider\APIextension\EntityManager;

use Magento\Framework\DataObject;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Sales\Api\Data\{
    OrderInterface,
    OrderExtensionInterface,
    OrderAddressInterface
};
use SyncSpider\APIextension\EntityManager\OrderAddressReadHandler;
use SyncSpider\APIextension\Model\ExtensionAttributes\Processor\OrderAddressExtensionAttributesProcessor;

class OrderReadHandler implements ExtensionInterface
{
    /**
     * @var []
     */
    private $map;

    /**
     * @var OrderAddressReadHandler
     */
    private $orderAddressReadHandler;

    /**
     * @var OrderAddressExtensionAttributesProcessor
     */
    private $addressExtensionProcessor;

    public function __construct(
        OrderAddressReadHandler $orderAddressReadHandler,
        OrderAddressExtensionAttributesProcessor $addressExtensionProcessor,
        array $map = []
    ) {
        $this->orderAddressReadHandler = $orderAddressReadHandler;
        $this->addressExtensionProcessor = $addressExtensionProcessor;
        $this->map = $map;
    }

    /**
     * Assign order entity column values to its extension attributes
     *
     * Utilizes the "map" property to map order entity column value to its extension attribute value
     *
     * @param OrderItemInterface $entity
     * @param array $arguments
     *
     * @return OrderItemInterface
     */
    public function execute($entity, $arguments = [])
    {
        if ($entity instanceof OrderInterface && $entity instanceof DataObject) {
            $extensionAttributes = $entity->getExtensionAttributes();

            if ($extensionAttributes instanceof OrderExtensionInterface) {
                foreach ($this->map as $column => $extensionAttribute) {
                    $value = $entity->getData($column);
                    $setter = SimpleDataObjectConverter::snakeCaseToCamelCase('set_' . $extensionAttribute);

                    if (! empty($value) && method_exists($extensionAttributes, $setter)) {
                        $extensionAttributes->{$setter}($value);
                    }
                }
            }

            $this->processAddresses($entity);
        }

        return $entity;
    }

    /**
     * Process given order addreses
     *
     * @param OrderInterface $order
     *
     * @return OrderInterface
     */
    public function processAddresses(OrderInterface $order): OrderInterface
    {
        $billingAddress = $order->getBillingAddress();
        $shippingAddress = $this->addressExtensionProcessor->getShippingAddress($order);

        if ($billingAddress instanceof OrderAddressInterface) {
            $this->orderAddressReadHandler->execute($billingAddress, []);
        }

        if ($shippingAddress instanceof OrderAddressInterface) {
            $this->orderAddressReadHandler->execute($shippingAddress, []);
        }

        return $order;
    }
}
