<?php

namespace SyncSpider\APIextension\EntityManager;

use Magento\Framework\DataObject;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Sales\Api\Data\{OrderInterface, OrderExtensionInterface};

class OrderSaveHandler implements ExtensionInterface
{
    /**
     * @var []
     */
    private $map;

    public function __construct(array $map = [])
    {
        $this->map = $map;
    }

    /**
     * Assign order entity extension attributes to its column values
     *
     * Utilizes the "map" property to map order entity extension attribute value to its column value 
     *
     * @param OrderItemInterface $entity
     * @param array $arguments
     *
     * @return OrderItemInterface
     */
    public function execute($entity, $arguments = [])
    {
        if ($entity instanceof OrderInterface && $entity instanceof DataObject) {
            $extensionAttributes = $entity->getExtensionAttributes();

            if ($extensionAttributes instanceof OrderExtensionInterface) {
                foreach ($this->map as $extensionAttribute => $column) {
                    $getter = SimpleDataObjectConverter::snakeCaseToCamelCase('get_' . $extensionAttribute);

                    if (method_exists($extensionAttributes, $getter)) {
                        $value = $extensionAttributes->{$getter}();

                        if (! empty($value)) {
                            $entity->setData($column, $value);
                        }
                    }
                }
            }
        }

        return $entity;
    }
}