<?php

namespace SyncSpider\APIextension\EntityManager\Swatches;

use Magento\Framework\DataObject;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Eav\Api\Data\AttributeInterface;
use Magento\Catalog\Model\ResourceModel\Eav\Attribute as CatalogAttribute;
use Magento\Swatches\Helper\Data as SwatchHelper;
use SyncSpider\APIextension\Api\UploaderInterface;

class AttributeSaveHandler implements ExtensionInterface
{
    /**
     * @var SwatchHelper
     */
    private $swatchHelper;

    /**
     * @var UploaderInterface
     */
    private $uploader;

    public function __construct(
        SwatchHelper $swatchHelper,
        UploaderInterface $uploader
    ) {
        $this->swatchHelper = $swatchHelper;
        $this->uploader = $uploader;
    }

    public function execute($entity, $arguments = [])
    {
        if ($entity instanceof CatalogAttribute
            && $entity instanceof DataObject
            && $this->swatchHelper->isVisualSwatch($entity)
        ) {
            $arraySwatch = $entity->getData('swatch');
            $arraySwatchVisual = $entity->getData('swatchvisual');
            $valuesSwatch = $arraySwatch['value'] ?? [];
            $valuesSwatchVisual = $arraySwatchVisual['value'] ?? [];

            // process visual swatches
            if (is_array($valuesSwatchVisual)) {
                $processValueSwatch = is_array($valuesSwatch);
                $valuesSwatchVisual = $this->extractResourceUrlValues($valuesSwatchVisual);

                // process visual swatches resource urls
                foreach ($valuesSwatchVisual as $id => $sourceUrl) {
                    $swatchPath = $this->uploader->saveSwatchImage($sourceUrl);

                    // update option value from resource url to swatch image path
                    if ($swatchPath) {
                        $valuesSwatchVisual[$id] = $swatchPath;

                        if ($processValueSwatch && array_key_exists($id, $valuesSwatch)) {
                            $valuesSwatch[$id] = $swatchPath;
                        }
                    }
                }

                // persist updated values to entity so they are saved
                if (! empty($valuesSwatchVisual)) {
                    $arraySwatchVisual['value'] = $valuesSwatchVisual;
                    $entity->setData('swatchvisual', $arraySwatchVisual);
                }

                if ($processValueSwatch && ! empty($valuesSwatch)) {
                    $arraySwatch['value'] = $valuesSwatch;
                    $entity->setData('swatch', $arraySwatch);
                }
            }
        }

        return $entity;
    }

    /**
     * Extract resource url values from given option value map
     *
     * @param array $optionValueMap
     *
     * @return array
     */
    public function extractResourceUrlValues(array $optionValueMap): array
    {
        $optionValueMap = array_filter($optionValueMap, function ($value) {
            return filter_var($value, FILTER_VALIDATE_URL);
        });
        return $optionValueMap;
    }
}