<?php

namespace SyncSpider\APIextension\Model\Api;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Shipping\Model\Config;
use Magento\Store\Model\StoreManagerInterface;
use SyncSpider\APIextension\Api\Carrier\CarrierRepositoryInterface;
use SyncSpider\APIextension\Api\Carrier\CarrierSearchResultsInterface;
use SyncSpider\APIextension\Api\Carrier\CarrierSearchResultsInterfaceFactory;

class CarrierRepository implements CarrierRepositoryInterface
{
    /**
     * @var CarrierSearchResultsInterfaceFactory
     */
    private $searchFactory;
    /**
     * @var Config
     */
    private $shippingConfig;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * CarrierRepository constructor.
     *
     * @param CarrierSearchResultsInterfaceFactory $searchFactory
     * @param StoreManagerInterface $storeManager
     * @param Config $shippingConfig
     */
    public function __construct(
        CarrierSearchResultsInterfaceFactory $searchFactory,
        StoreManagerInterface $storeManager,
        Config $shippingConfig
    ) {
        $this->searchFactory = $searchFactory;
        $this->shippingConfig = $shippingConfig;
        $this->storeManager = $storeManager;
    }

    /**
     * Gets available currencies.
     *
     * @api
     *
     * @param SearchCriteriaInterface $searchCriteria
     *
     * @return CarrierSearchResultsInterface
     */
    public function get(SearchCriteriaInterface $searchCriteria)
    {
        $items = $this->getCarriers();

        $searchResult = $this->searchFactory->create();
        $searchResult->setSearchCriteria($searchCriteria);
        $searchResult->setItems($items);
        $searchResult->setTotalCount(count($items));

        return $searchResult;
    }

    /**
     * Retrieve carriers
     *
     * @return array
     */
    public function getCarriers()
    {
        $carriers = [
            [
                'value' => 'custom',
                'label' => __('Custom Value')
            ]
        ];

        $carrierInstances = $this->getCarriersInstances();
        foreach ($carrierInstances as $code => $carrier) {
            if ($carrier->isTrackingAvailable()) {
                $carriers[] = [
                    'value' => $code,
                    'label' => $carrier->getConfigData('title')
                ];
            }
        }

        return $carriers;
    }

    /**
     * Get carrier instances for current store
     *
     * @return array
     */
    private function getCarriersInstances()
    {
        $store = $this->storeManager->getStore();
        return $this->shippingConfig->getAllCarriers($store->getId());
    }
}
