<?php

namespace SyncSpider\APIextension\Model\Api;

use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Store\Model\StoreManagerInterface;
use SyncSpider\APIextension\Api\CategoryRepositoryInterface;
use SyncSpider\APIextension\Api\Data\CategorySearchResultsInterfaceFactory;

class CategoryRepository implements CategoryRepositoryInterface
{
    /**
     * @var CategorySearchResultsInterfaceFactory
     */
    private $searchFactory;
    /**
     * @var CollectionFactory
     */
    private $collectionFactory;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * CategoryRepository constructor.
     *
     * @param CollectionFactory $collectionFactory
     * @param StoreManagerInterface $storeManager
     * @param CategorySearchResultsInterfaceFactory $searchFactory
     */
    public function __construct(
        CollectionFactory $collectionFactory,
        StoreManagerInterface $storeManager,
        CategorySearchResultsInterfaceFactory $searchFactory
    ) {
        $this->collectionFactory = $collectionFactory;
        $this->storeManager = $storeManager;
        $this->searchFactory = $searchFactory;
    }

    /**
     * Retrieves categories
     *
     * @api
     *
     * @param SearchCriteriaInterface $searchCriteria
     *
     * @return \SyncSpider\APIextension\Api\Data\CategorySearchResultsInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getCategories(SearchCriteriaInterface $searchCriteria)
    {
        $collection = $this->collectionFactory->create();
        $collection->addAttributeToSelect('*');

        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        $collection->load();

        $searchResult = $this->searchFactory->create();
        $searchResult->setSearchCriteria($searchCriteria);
        $searchResult->setItems($collection->getItems());
        $searchResult->setTotalCount($collection->getSize());

        return $searchResult;
    }
}
