<?php

namespace SyncSpider\APIextension\Model\Api;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Locale\CurrencyInterface;
use Magento\Store\Api\StoreRepositoryInterface;
use SyncSpider\APIextension\Api\CurrencyRepositoryInterface;
use SyncSpider\APIextension\Api\Data\CurrencySearchResultsInterfaceFactory;

class CurrencyRepository implements CurrencyRepositoryInterface
{
    /**
     * @var CurrencySearchResultsInterfaceFactory
     */
    private $searchFactory;
    /**
     * @var StoreRepositoryInterface
     */
    private $repository;
    /**
     * @var CurrencyInterface
     */
    private $locale;

    /**
     * CategoryRepository constructor.
     *
     * @param CurrencySearchResultsInterfaceFactory $searchFactory
     * @param StoreRepositoryInterface $repository
     * @param CurrencyInterface $locale
     */
    public function __construct(
        CurrencySearchResultsInterfaceFactory $searchFactory,
        StoreRepositoryInterface $repository,
        CurrencyInterface $locale
    ) {
        $this->searchFactory = $searchFactory;
        $this->repository = $repository;
        $this->locale = $locale;
    }

    /**
     * Gets available currencies
     *
     * @api
     *
     * @param SearchCriteriaInterface $searchCriteria
     *
     * @return \SyncSpider\APIextension\Api\Data\CurrencySearchResultsInterface
     */
    public function get(SearchCriteriaInterface $searchCriteria)
    {
        $items = [];
        /** @var \Magento\Store\Model\Store $store */
        foreach ($this->repository->getList() as $store) {
            foreach ($store->getAvailableCurrencyCodes() as $code) {
                if (!array_key_exists($code, $items)) {
                    $items[$code] = [
                        'code' => $code,
                        'name' => $this->locale->getCurrency($code)->getName(),
                    ];
                }
            }
        }

        $searchResult = $this->searchFactory->create();
        $searchResult->setSearchCriteria($searchCriteria);
        $searchResult->setItems($items);
        $searchResult->setTotalCount(count($items));

        return $searchResult;
    }
}
