<?php

namespace SyncSpider\APIextension\Model\Api;

use Exception;
use SyncSpider\APIextension\Api\Data\OrderInterface;
use SyncSpider\APIextension\Api\OrderRepositoryInterface;
use SyncSpider\APIextension\Api\InventoryStockServiceInterface;

class OrderRepository implements OrderRepositoryInterface
{
    /**
     * @var \SyncSpider\APIextension\Model\Order\Order
     */
    private $order;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \SyncSpider\APIextension\Model\Order\Validation
     */
    private $validation;

    /**
     * @var InventoryStockServiceInterface
     */
    private $stockService;

    /**
     * OrderRepository constructor.
     *
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \SyncSpider\APIextension\Model\Order\Order $order
     * @param \SyncSpider\APIextension\Model\Order\Validation $validation
     * @param \Magento\Catalog\Helper\Product $productHelper
     * @param InventoryStockServiceInterface $stockService
     */
    public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \SyncSpider\APIextension\Model\Order\Order $order,
        \SyncSpider\APIextension\Model\Order\Validation $validation,
        InventoryStockServiceInterface $stockService
    ) {
        $this->order = $order;
        $this->validation = $validation;
        $this->storeManager = $storeManager;
        $this->stockService = $stockService;
    }

    /**
     * @inheritDoc
     */
    public function save(OrderInterface $order)
    {
        // execute order creation function within "backorder" inventory stock feature context
        return $this->stockService->backorders()
            ->execute([$this, 'createOrder'], [$order]);
    }

    /**
     * Create magento 2 order from given syncspider order
     *
     * Syncspider order is first converted to quote and placed using
     * admin area order creation logic
     *
     * @param OrderInterface $order
     *
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    public function createOrder(OrderInterface $order)
    {
        $store = $this->storeManager->getStore();

        try {
            $this->validation->validate($order, $store);
            $newOrder = $this->order->create($order, $store);
        } catch (Exception $e) {
            throw new \Magento\Framework\Exception\LocalizedException(__($e->getMessage()));
        }

        return $newOrder;
    }
}
