<?php

namespace SyncSpider\APIextension\Model\Data;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\Model\AbstractExtensibleModel;
use SyncSpider\APIextension\Api\Data\{
    OrderInterface,
    OrderPaymentInterface
};
use Magento\Sales\Api\Data\{
    OrderInterface as MagentoOrderInterface,
    AddressExtensionInterface
};
use SyncSpider\APIextension\Api\Data\OrderCustomerAddressInterface;

class Order extends AbstractExtensibleModel implements OrderInterface
{
    /**
     * @var \SyncSpider\APIextension\Api\Data\OrderItemInterface[]
     */
    private $products = [];

    /**
     * @var \SyncSpider\APIextension\Api\Data\OrderCustomerInterface
     */
    private $customer;

    /**
     * @var \SyncSpider\APIextension\Api\Data\OrderCustomerAddressInterface
     */
    private $billingAddress = [];

    /**
     * @var \SyncSpider\APIextension\Api\Data\OrderCustomerAddressInterface
     */
    private $shippingAddress = [];

    /**
     * @var \SyncSpider\APIextension\Api\Data\OrderPaymentInterface
     */
    private $payment = [];

    /**
     * @var \SyncSpider\APIextension\Api\Data\OrderShippingInterface
     */
    private $shipping;

    /**
     * @var array
     */
    private $account = [];

    /**
     * @inheritDoc
     */
    public function getIncrementId()
    {
        return $this->getData('increment_id');
    }

    /**
     * @inheritDoc
     */
    public function setIncrementId($incrementId)
    {
        $this->setData('increment_id', $incrementId);
    }

    /**
     * @inheritDoc
     */
    public function getCurrencyCode()
    {
        return $this->getData('currency_code');
    }

    /**
     * @inheritDoc
     */
    public function setCurrencyCode($currencyCode)
    {
        $this->setData('currency_code', $currencyCode);
    }

    /**
     * @inheritDoc
     */
    public function getStatus()
    {
        return $this->getData('status');
    }

    /**
     * @inheritDoc
     */
    public function setStatus($status)
    {
        $this->setData('status', $status);
    }

    /**
     * @inheritDoc
     */
    public function getProducts()
    {
        return $this->products;
    }

    /**
     * @inheritDoc
     */
    public function setProducts($products)
    {
        $this->products = $products;
    }

    /**
     * @inheritDoc
     */
    public function getCustomer()
    {
        return $this->customer;
    }

    /**
     * @inheritDoc
     */
    public function setCustomer($customer)
    {
        $this->customer = $customer;
    }

    /**
     * @inheritDoc
     */
    public function getBillingAddress()
    {
        return $this->billingAddress;
    }

    /**
     * @inheritDoc
     */
    public function setBillingAddress($billingAddress)
    {
        $this->billingAddress = $billingAddress;
    }

    /**
     * @inheritDoc
     */
    public function getShippingAddress()
    {
        return $this->shippingAddress;
    }

    /**
     * @inheritDoc
     */
    public function setShippingAddress($shippingAddress)
    {
        $this->shippingAddress = $shippingAddress;
    }

    /**
     * @inheritDoc
     */
    public function getPayment()
    {
        return $this->payment;
    }

    /**
     * @inheritDoc
     */
    public function setPayment($payment)
    {
        $this->payment = $payment;
    }

    /**
     * @inheritDoc
     */
    public function getShipping()
    {
        return $this->shipping;
    }

    /**
     * @inheritDoc
     */
    public function setShipping($shipping)
    {
        $this->shipping = $shipping;
    }

    /**
     * @inheritDoc
     */
    public function getShippingTaxClassId()
    {
        return $this->getData('shipping_tax_class_id');
    }

    /**
     * @inheritDoc
     */
    public function setShippingTaxClassId($shippingTaxId)
    {
        $this->setData('shipping_tax_class_id', $shippingTaxId);
    }

    /**
     * @inheritDoc
     */
    public function getComment()
    {
        return $this->getData('comment');
    }

    /**
     * @inheritDoc
     */
    public function setComment($comment)
    {
        $this->setData('comment', $comment);
    }

    /**
     * @inheritDoc
     */
    public function getCouponCode()
    {
        return $this->getData('coupon_code');
    }

    /**
     * @inheritDoc
     */
    public function setCouponCode($couponCode)
    {
        $this->setData('coupon_code', $couponCode);
    }

    /**
     * @inheritDoc
     */
    public function isPriceIncludeTax()
    {
        return $this->getData('price_include_tax');
    }

    /**
     * @inheritDoc
     */
    public function setPriceIncludeTax($priceIncludeTax)
    {
        $this->setData('price_include_tax', $priceIncludeTax);
    }

    /**
     * @inheritDoc
     */
    public function isCreateCustomer()
    {
        return $this->getData('create_customer');
    }

    /**
     * @inheritDoc
     */
    public function setCreateCustomer($createCustomer)
    {
        $this->setData('create_customer', $createCustomer);
    }

    /**
     * @inheritDoc
     */
    public function isSameAsBilling()
    {
        return $this->getData('same_as_billing') === true;
    }

    /**
     * @inheritDoc
     */
    public function setSameAsBilling($sameAsBilling)
    {
        $this->setData('same_as_billing', $sameAsBilling);
    }

    /**
     * @inheritDoc
     */
    public function isCreateProducts()
    {
        return $this->getData('create_products');
    }

    /**
     * @inheritDoc
     */
    public function setCreateProducts($createProducts)
    {
        $this->setData('create_products', $createProducts);
    }

    /**
     * @inheritDoc
     */
    public function getShippingMethod()
    {
        return 'syncspider_syncspider';
    }

    /**
     * @inheritDoc
     */
    public function getPaymentMethod()
    {
        return \SyncSpider\APIextension\Api\PaymentInterface::CODE;
    }

    /**
     * @inheritDoc
     */
    public function getCustomerEmail()
    {
        $customer = $this->getCustomer();
        return isset($customer['email']) ? $customer['email'] : null;
    }

    /**
     * @inheritDoc
     */
    public function getCustomerGroupId()
    {
        return $this->getCustomer()->getGroupId();
    }

    /**
     * @inheritDoc
     */
    public function isGuestCustomer()
    {
        return $this->getCustomer()->isGuest() === true;
    }

    /**
     * @inheritDoc
     */
    public function shouldCreateCustomer()
    {
        $customer = $this->getCustomer();
        $isCreateCustomerEnabled = $this->isCreateCustomer();
        return $customer !== null && isset($isCreateCustomerEnabled) && $isCreateCustomerEnabled === true;
    }

    /**
     * @inheritDoc
     */
    public function shouldCreateProduct()
    {
        return $this->isCreateProducts() === true;
    }

    /**
     * @inheritDoc
     */
    public function getAccount()
    {
        return $this->account;
    }

    /**
     * @inheritDoc
     */
    public function setAccount($account)
    {
        $this->account = $account;
    }


    /**
     * @inheritDoc
     */
    public function getExtOrderId()
    {
        return $this->getPayment() ? $this->getPayment()->getExternalOrderId() : null;
    }

    /**
     * @inheritDoc
     */
    public function setExtOrderId($id)
    {
        $payment = $this->getPayment();

        if (! $payment) {
            $payment = ObjectManager::getInstance()->create(OrderPaymentInterface::class);
            $this->setPayment($payment);
        }

        $payment->setExternalOrderId($id);
        return $this;
    }

    /**
     * @inheritDoc
     *
     * @return \Magento\Sales\Api\Data\OrderExtensionInterface|null
     */
    public function getExtensionAttributes()
    {
        if (! $this->getData(self::EXTENSION_ATTRIBUTES_KEY)) {
            $this->populateExtensionAttributes([]);
        }

        return $this->_getExtensionAttributes();
    }

    /**
     * @inheritDoc
     *
     * @param \Magento\Sales\Api\Data\OrderExtensionInterface $extensionAttributes
     * @return $this
     */
    public function setExtensionAttributes(\Magento\Sales\Api\Data\OrderExtensionInterface $extensionAttributes)
    {
        return $this->_setExtensionAttributes($extensionAttributes);
    }

    /**
     * Instantiate extension attributes object and populate it with the provided data.
     *
     * @param array $extensionAttributesData
     * @return void
     */
    private function populateExtensionAttributes(array $extensionAttributesData = [])
    {
        $extensionAttributes = $this->extensionAttributesFactory->create(MagentoOrderInterface::class, $extensionAttributesData);
        $this->_setExtensionAttributes($extensionAttributes);
    }

    /**
     * Convert given addrss to array
     *
     * Remove extension attributes in case they are not in required format
     *
     * @param OrderCustomerAddressInterface $address
     *
     * @return array
     */
    public function convertAddressToArray(OrderCustomerAddressInterface $address): array
    {
        $data =  $address->toArray();

        if (array_key_exists('extension_attributes', $data)
            && ! $data['extension_attributes'] instanceof AddressExtensionInterface
        ) {
            unset($data['extension_attributes']);
        }

        return $data;
    }

    /**
     * @inheritDoc
     */
    public function toArray(array $keys = [])
    {
        return [
            'increment_id' => $this->getIncrementId(),
            'currency_code' => $this->getCurrencyCode(),
            'billing_address' => $this->getBillingAddress() ? $this->convertAddressToArray($this->getBillingAddress()) : null,
            'shipping_address' => $this->getShippingAddress() ? $this->convertAddressToArray($this->getShippingAddress()) : null,
            'shipping_tax_id' => $this->getShippingTaxClassId(),
            'shipping_method' => $this->getShippingMethod(),
            'same_as_billing' => $this->isSameAsBilling(),
            'payment_method' => $this->getPaymentMethod(),
            'coupon' => ['code' => $this->getCouponCode()],
            'comment' => ['customer_note' => $this->getComment()],
            'products' => $this->getProducts(),
            'customer' => $this->getCustomer() ? $this->getCustomer()->toArray() : null,
            'payment' => $this->getPayment() ? $this->getPayment()->toArray() : [],
            'account' => $this->getAccount(),
            'send_confirmation' => false,
            'ext_order_id' => $this->getExtOrderId()
        ];
    }
}
