<?php

namespace SyncSpider\APIextension\Model\ExtensionAttributes\Processor;

use Magento\Framework\Api\ExtensibleDataInterface;
use SyncSpider\APIextension\Api\{
    Data\SimpleOptionInterfaceFactory,
    ExtensionAttributes\ProcessorContextInterface,
    ExtensionAttributes\ProcessorInterface
};
use SyncSpider\APIextension\Model\ProductOption\{
    InformationResolverInterface,
    ProductOptionInformation
};
use Magento\Framework\DataObject;

class AttributeInformationProcessor implements ProcessorInterface
{
    /**
     * @var InformationResolverInterface[]
     */
    private $informationResolverPool;

    /**
     * @var SimpleOptionInterfaceFactory
     */
    private $simpleOptionFactory;

    public function __construct(SimpleOptionInterfaceFactory $simpleOptionFactory, array $informationResolverPool)
    {
        $this->simpleOptionFactory = $simpleOptionFactory;
        $this->informationResolverPool = $informationResolverPool;
    }

    /**
     * @inheritDoc
     */
    public function process(ExtensibleDataInterface $entity, ProcessorContextInterface $context): ExtensibleDataInterface
    {
        $request = $context->getContextValue(ProductOptionInformation::REQUEST_CONTEXT);

        if ($request instanceof DataObject) {
            foreach ($this->informationResolverPool as $informationResolver) {
                if ($informationResolver->validEntity($entity)) {
                    $keyFields = $informationResolver->entityInformationKeyFields($entity);
                    $optionInformation = $informationResolver->getProductOptionInformation($request);
                    $informationKey = $optionInformation->getInformationKey($keyFields);
                    $informationMap = $optionInformation->getInformationMap();
    
                    if (isset($informationMap[$informationKey])) {
                        $extensionAttributes = $context->getEntityExtensionAttributes($entity);
    
                        if (method_exists($extensionAttributes, 'setSyncspiderAttributeInfo')) {
                            $simpleOption = $this->simpleOptionFactory->create([
                                'data' => $informationMap[$informationKey]
                            ]);            
                            $extensionAttributes->setSyncspiderAttributeInfo($simpleOption);
                        }
                    }
                }
            }
        }

        return $entity;
    }
}
