<?php

namespace SyncSpider\APIextension\Model\ExtensionAttributes\Processor;

use Magento\Framework\Api\ExtensibleDataInterface;
use Magento\Sales\Api\Data\OrderInterface;
use SyncSpider\APIextension\Api\{
    Data\OrderPaymentInterface as SyncSpiderOrderPaymentInterface,
    ExtensionAttributes\ProcessorContextInterface,
    ExtensionAttributes\ProcessorInterface
};
use SyncSpider\APIextension\Model\ExtensionAttributes\SalesOrderContext;

class OrderExtOrderIdProcessor implements ProcessorInterface
{
    /**
     * @var SalesOrderContext
     */
    private $orderContext;

    public function __construct(SalesOrderContext $orderContext)
    {
        $this->orderContext = $orderContext;
    }

    /**
     * Populate order "ext_order_id" field
     *
     * Value is set to syncspider order peyment external order id
     *
     * @param ExtensibleDataInterface $entity
     * @param ProcessorContextInterface $context
     *
     * @return ExtensibleDataInterface
     */
    public function process(ExtensibleDataInterface $entity, ProcessorContextInterface $context): ExtensibleDataInterface
    {
        if ($entity instanceof OrderInterface) {
            $syncspiderOrder = $this->orderContext->getSyncspiderOrder($context);

            if ($syncspiderOrder) {
                $payment = $syncspiderOrder->getPayment();

                if ($payment instanceof SyncSpiderOrderPaymentInterface
                    && $payment->getExternalOrderId()
                ) {
                    $entity->setExtOrderId($payment->getExternalOrderId());
                }
            }
        }

        return $entity;
    }
}