<?php

namespace SyncSpider\APIextension\Model\ExtensionAttributes\Processor;

use Magento\Framework\Api\{
    AbstractSimpleObject,
    ExtensibleDataInterface,
    SimpleDataObjectConverter
};
use Magento\Quote\Api\Data\CartInterface;
use Magento\Sales\Api\Data\OrderInterface;
use SyncSpider\APIextension\Api\{
    ExtensionAttributes\ProcessorContextInterface,
    ExtensionAttributes\ProcessorInterface
};
use SyncSpider\APIextension\Model\ExtensionAttributes\SalesOrderContext;

class QuoteOrderExtensionAttributesProcessor implements ProcessorInterface
{
    /**
     * @var SalesOrderContext
     */
    private $orderContext;

    public function __construct(SalesOrderContext $orderContext)
    {
        $this->orderContext = $orderContext;
    }

    /**
     * Process quote cart entity extension attributes
     *
     * Merge quote cart entity "syncspider_order_extension" extension attribute
     * into sales order entity extension attributes
     *
     * @param OrderInterface $order
     * @param CartInterface $quote
     *
     * @return OrderInterface
     */
    public function process(ExtensibleDataInterface $entity, ProcessorContextInterface $context): ExtensibleDataInterface
    {
        if ($entity instanceof OrderInterface) {
            $extensionAttributes = $context->getEntityExtensionAttributes($entity);
            $orderExtension = $this->orderContext->getOrderExtension($context);

            if ($orderExtension instanceof AbstractSimpleObject) {
                $extensionData = $orderExtension->__toArray();
                
                foreach ($extensionData as $extensionAttribute => $_) {
                    $setter = SimpleDataObjectConverter::snakeCaseToCamelCase('set_' . $extensionAttribute);
                    $getter = SimpleDataObjectConverter::snakeCaseToCamelCase('get_' . $extensionAttribute);

                    if (method_exists($extensionAttributes, $setter)
                        && method_exists($extensionAttributes, $getter)
                    ) {
                        $extensionAttributes->{$setter}($orderExtension->{$getter}());
                    }
                }
            }
        }

        return $entity;
    }
}
