<?php

namespace SyncSpider\APIextension\Model\ExtensionAttributes;

use Magento\Framework\Api\{
    ExtensibleDataInterface,
    ExtensionAttributesInterface,
    ExtensionAttributesFactory
};
use SyncSpider\APIextension\Api\ExtensionAttributes\ProcessorContextInterface;

class ProcessorContext implements ProcessorContextInterface
{
    /**
     * @var ExtensionAttributesFactory
     */
    private $extensionAttributesFactory;

    /**
     * @var array
     */
    private $context;

    public function __construct(
        ExtensionAttributesFactory $extensionAttributesFactory,
        array $context = []
    ) {
        $this->extensionAttributesFactory = $extensionAttributesFactory;
        $this->context = $context;
    }

    /**
     * @inheritDoc
     */
    public function setContext(array $context): ProcessorContextInterface
    {
        $this->context = $context;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function retrieveContext(): array
    {
        return $this->context;
    }

    /**
     * @inheritDoc
     */
    public function validEntity($entity): bool
    {
        return is_object($entity)
            && $entity instanceof ExtensibleDataInterface
            && method_exists($entity, 'getExtensionAttributes')
            && method_exists($entity, 'setExtensionAttributes');
    }

    /**
     * @inheritDoc
     */
    public function getEntityExtensionAttributes(ExtensibleDataInterface $entity): ExtensionAttributesInterface
    {
        $extensionAttributes = $entity->getExtensionAttributes();

        if (! $extensionAttributes instanceof ExtensionAttributesInterface) {
            $extensionAttributes = $this->extensionAttributesFactory->create(get_class($entity));
            $entity->setExtensionAttributes($extensionAttributes);
        }

        return $extensionAttributes;
    }

    /**
     * @inheritDoc
     */
    public function setContextValue(string $context, $value): ProcessorContextInterface
    {
        $this->context[$value] = $context;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getContextValue(string $context)
    {
        return isset($this->context[$context]) ? $this->context[$context] : null;
    }
}