<?php

namespace SyncSpider\APIextension\Model;

use SyncSpider\APIextension\Api\FilterMetadataInterface;

class FilterMetadata implements FilterMetadataInterface
{
    /**
     * @var string
     */
    private $tableName;

    /**
     * @var string
     */
    private $tableAlias;

    /**
     * @var string
     */
    private $fieldPrefix;

    /**
     * @var string
     */
    private $constraint;

    /**
     * @var string]null
     */
    private $column;

    /**
     * FilterMetadata constructor.
     *
     * @param string $tableName
     * @param string $tableAlias
     * @param string $fieldPrefix
     * @param string $constraint
     * @param string $column
     */
    public function __construct(
        string $tableName,
        string $tableAlias,
        string $fieldPrefix,
        string $constraint,
        string $column = ''
    ) {
        $this->tableName = $tableName;
        $this->tableAlias = $tableAlias;
        $this->fieldPrefix = $fieldPrefix;
        $this->constraint = $constraint;
        $this->column = $column;
    }

    /**
     * @inheritDoc
     */
    public function getTableName(): string
    {
        return $this->tableName;
    }

    /**
     * @inheritDoc
     */
    public function getTableAlias(): string
    {
        return $this->tableAlias;
    }

    /**
     * @inheritDoc
     */
    public function getFieldPrefix(): string
    {
        return $this->fieldPrefix;
    }

    /**
     * @inheritDoc
     */
    public function getConstraint(): string
    {
        return $this->constraint;
    }

    /**
     * @inheritDoc
     */
    public function getColumn(): string
    {
        return $this->column;
    }

    /**
     * @inheritDoc
     */
    public function setTableName(string $tableName): FilterMetadataInterface
    {
        $this->tableName = $tableName;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function setTableAlias(string $tableAlias): FilterMetadataInterface
    {
        $this->tableAlias = $tableAlias;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function setFieldPrefix(string $fieldPrefix): FilterMetadataInterface
    {
        $this->fieldPrefix = $fieldPrefix;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function setConstraint(string $constraint): FilterMetadataInterface
    {
        $this->constraint = $constraint;
        return $this;
    }
}
