<?php

namespace SyncSpider\APIextension\Model;

use Magento\Framework\Api\Filter;
use Magento\Framework\DB\Select;
use SyncSpider\APIextension\Api\FilterMetadataInterface;
use SyncSpider\APIextension\Api\FilterRendererInterface;
use Exception;

class FilterRenderer implements FilterRendererInterface
{
    /**
     * Get bind name
     *
     * @param string $suffix
     *
     * @return string
     */
    public function bindName(string $suffix): string
    {
        return 'bind' . $suffix;
    }

    /**
     * @inheritDoc
     */
    public function render(
        Filter $filter,
        Select $select,
        FilterMetadataInterface $filterMetadata
    ): string {
        $condition = $filter->getConditionType() ?? 'eq';
        $values = [
            'eq'            => "{{fieldName}} = ?",
            'neq'           => "{{fieldName}} != ?",
            'like'          => "{{fieldName}} LIKE ?",
            'nlike'         => "{{fieldName}} NOT LIKE ?",
            'in'            => "{{fieldName}} IN(?)",
            'nin'           => "{{fieldName}} NOT IN(?)",
            'is'            => "{{fieldName}} IS ?",
            'notnull'       => "{{fieldName}} IS NOT NULL",
            'null'          => "{{fieldName}} IS NULL",
            'gt'            => "{{fieldName}} > ?",
            'lt'            => "{{fieldName}} < ?",
            'gteq'          => "{{fieldName}} >= ?",
            'lteq'          => "{{fieldName}} <= ?",
            'finset'        => "FIND_IN_SET(?, {{fieldName}})",
            'nfinset'       => "NOT FIND_IN_SET(?, {{fieldName}})",
            'regexp'        => "{{fieldName}} REGEXP ?",
            'from'          => "{{fieldName}} >= ?",
            'to'            => "{{fieldName}} <= ?",
            'seq'           => null,
            'sneq'          => null,
            'ntoa'          => "INET_NTOA({{fieldName}}) LIKE ?"
        ];

        if (! isset($values[$condition])) {
            throw new Exception('Unhandled condition type given "' . $condition . '"');
        }

        $binds = [];
        $bindNames = [];
        $suffix = ($select->getBind()[static::SUFFIX_BIND_NAME] ?? 0) + count($select->getBind());
        $filterValue = $filter->getValue();

        // convert in and nin filter value to array if necessary
        if (in_array($condition, ['in', 'nin']) && ! is_array($filterValue)) {
            $filterValue = explode(',', $filterValue);
        } else {
            // by default convert all filter values to array so they have the same bind name processing
            $filterValue = [(string) $filterValue];
        }

        foreach ($filterValue as $key => $value) {
            $bindName = $this->bindName($suffix . $key);
            $binds[$bindName] = $value;
            $bindNames[] = ':' . $bindName;
        }

        // replace quoted field name with table name
        $condition = str_replace(
            '{{fieldName}}',
            $select->getConnection()->quoteIdentifier(
                str_replace(
                    $filterMetadata->getFieldPrefix(),
                    $filterMetadata->getTableAlias() . '.' . $filterMetadata->getColumn(),
                    $filter->getField()
                )
            ),
            $values[$condition]
        );
        // replace value with bind parameters
        $condition = str_replace(
            '?',
            implode(',', $bindNames),
            $condition
        );
        // merge filter bind parameters with select binds
        $select->bind(array_merge($select->getBind(), $binds));
        return $condition;
    }
}
