<?php

namespace SyncSpider\APIextension\Model\ObjectManager;

class FactoryProcessor
{
    /**
     * @var \Magento\Framework\ObjectManager\ConfigInterface
     */
    private $config;

    /**
     * @var array
     */
    private $interfaceDefaults;

    /**
     * ConfigPlugin constructor
     *
     * @param \Magento\Framework\ObjectManager\ConfigInterface $config
     * @param array $interfaceDefaults List of interface/class defaults in case magento can't find interface/class preference mainly for magento 2.1.x
     */
    public function __construct(
        \Magento\Framework\ObjectManager\ConfigInterface $config,
        array $interfaceDefaults
    ) {
        $this->config = $config;
        $this->interfaceDefaults = $interfaceDefaults;
    }

    /**
     * Fallback to additional interface/class default classes in case magento can't find interface/class preference
     *
     * @param string $instanceName
     * @return mixed
     */
    public function getInstanceType($instanceName)
    {
        $instanceName = ltrim($instanceName, '\\');
        $preference = $this->config->getPreference($instanceName);

        if (! class_exists($preference)) {
            if (isset($this->interfaceDefaults[$instanceName])
                && class_exists($this->interfaceDefaults[$instanceName])
            ) {
                return $this->interfaceDefaults[$instanceName];
            }
        }

        return $preference;
    }

    /**
     * Process factory instance and replace factory instance name if necessary
     *
     * @param mixed $factoryInstance
     * @return $this
     */
    public function process($factoryInstance)
    {
        if (is_object($factoryInstance) && property_exists($factoryInstance, '_instanceName')) {
            $property = new \ReflectionProperty($factoryInstance, '_instanceName');
            $property->setAccessible(true);
            $instanceName = $property->getValue($factoryInstance);
            $preference = $this->config->getPreference($instanceName);

            if (! class_exists($preference)) {
                $property->setValue($factoryInstance, $this->getInstanceType($instanceName));
            }
        }

        return $this;
    }
}
