<?php

namespace SyncSpider\APIextension\Model\Order;

class Configuration
{
    /**
     * @var array
     */
    private $cached = [];

    /**
     * Set cached configuration for given path and value
     *
     * @param string $path
     * @param mixed $value
     *
     * @return void
     */
    public function setValue($path, $value)
    {
        $this->cached[$path] = $value;
    }

    /**
     * Get cached configuration value for given path
     *
     * @param string|null $path
     *
     * @return mixed
     */
    public function getValue($path = null)
    {
        return isset($this->cached[$path]) ? $this->cached[$path] : null;
    }

    /**
     * Clear all cached configuration values
     *
     * @return void
     */
    public function clear()
    {
        $this->cached = [];
    }

    /**
     * Whether cached configuration changes exists
     *
     * @return bool
     */
    public function hasChanges()
    {
        return !empty($this->cached);
    }
}
