<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver;

use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolverContextInterface,
    EntriesResolverInterface
};

class CleanEntries implements EntriesResolverInterface
{
    /**
     * Clean given media gallery entries
     * 
     * - set position to match sequence
     * - removes the "content" from existing media gallery entries
     * so they are not saved again
     * - removes "file" from non existing media gallery entries to
     *  allow correct saving
     * - exclude new images in case of not create operation
     * - exclude "not fresh" syncspider images in case of create
     * operation
     *
     * {@inheritDoc}
     */
    public function getMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        $cleanEntries = [];
        $position = 1;

        foreach ($mediaGalleryEntries as $entry) {
            // clean existing entries
            if ($entry->getId()) {
                $syncspiderImage = $context->getSyncspiderImage($entry);

                if ($context->isPreservePreviousImages()
                    || (
                        ! $syncspiderImage || $syncspiderImage->getIsFresh()
                    )
                ) {
                    $cleanEntries[] = $entry;
                    $entry->setPosition($position++);

                    if ($entry->getContent()) {
                        $entry->setContent(null);
                    }
                }
            }

            // clean and include new entries in case of create operation
            if ($context->isCreateOperation()
                && ! $entry->getId()
                && $entry->getContent()
            ) {
                $entry->setFile(null);
                $cleanEntries[] = $entry;
                $entry->setPosition($position++);
            }
        }

        return $cleanEntries;
    }
}