<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver;

use Magento\Catalog\Api\Data\ProductAttributeMediaGalleryEntryInterface;
use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolverContextInterface,
    EntriesResolverInterface
};

class CleanEntriesTypes implements EntriesResolverInterface
{
    /**
     * Clean given media gallery entries types
     *
     * Correctly set media gallery entries types so each type can
     * be assigned only to one entry. Last entry has highest priority (LIFO)
     *
     * {@inheritDoc}
     */
    public function getMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        // utilize only valid types, so the last entry has highest priority for the type
        $usedTypes = [];

        for ($i = count($mediaGalleryEntries) - 1; $i >= 0;  $i--) {
            $entry = $mediaGalleryEntries[$i];
            $entryTypes = $entry->getTypes() ?? [];

            if (is_array($entryTypes)) {
                $entryTypes = array_diff($entryTypes, $usedTypes);
                $usedTypes = array_merge($usedTypes, $entryTypes);
                $entry->setTypes($entryTypes);
            }
        }

        return $mediaGalleryEntries;
    }

    /**
     * Clean types of given media gallery entries based on given main entry
     *
     * @param array $mediaGalleryEntries
     * @param ProductAttributeMediaGalleryEntryInterface $mainEntry
     *
     * @return array
     */
    public function cleanTypes(array $mediaGalleryEntries, ProductAttributeMediaGalleryEntryInterface $mainEntry): array
    {
        foreach ($mediaGalleryEntries as $entry) {
            if ($entry === $mainEntry) {
                continue;
            }

            $entry->setTypes(array_diff(
                $entry->getTypes() ?? [],
                $mainEntry->getTypes() ?? []
            ));
        }

        return $mediaGalleryEntries;
    }
}