<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver;

use Magento\Catalog\Api\Data\{
    ProductAttributeMediaGalleryEntryInterface,
    ProductAttributeMediaGalleryEntryExtensionInterface
};
use SyncSpider\APIextension\Api\Data\{
    SyncspiderImageInterface,
    SyncspiderImageInterfaceFactory
};
use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolverContextInterface,
    EntriesResolverInterface
};
use SyncSpider\APIextension\Model\ResourceModel\Image\CollectionFactory as ImageCollectionFactory;

class LoadSyncspiderImageData implements EntriesResolverInterface
{
    /**
     * @var ImageCollectionFactory
     */
    private $imageCollectionFactory;

    /**
     * @var SyncspiderImageInterfaceFactory
     */
    private $syncspiderImageFactory;

    public function __construct(
        ImageCollectionFactory $imageCollectionFactory,
        SyncspiderImageInterfaceFactory $syncspiderImageFactory
    ) {
        $this->imageCollectionFactory = $imageCollectionFactory;
        $this->syncspiderImageFactory = $syncspiderImageFactory;
    }

    /**
     * Get syncspider images for given eia gallery entries
     *
     * @param array $mediaGalleryEntries
     *
     * @return SyncspiderImageInterface[]
     */
    public function getSyncspiderImages(array $mediaGalleryEntries): array
    {
        $data = [];
        $imageIds = array_reduce(
            $mediaGalleryEntries,
            function (array $carry, ProductAttributeMediaGalleryEntryInterface $galleryImage) {
                if ($galleryImage->getId()) {
                    $carry[] = $galleryImage->getId();
                }

                return $carry;
            },
            []
        );

        if (! empty($imageIds)) {
            $syncspiderImages = $this->imageCollectionFactory->create()
                ->addFieldToFilter('image_id', ['in' => $imageIds])
                ->getItems();
            
            foreach ($syncspiderImages as $syncspiderImage) {
                $data[$syncspiderImage->getData('image_id')] = $this->syncspiderImageFactory->create([
                    'data' => [
                        SyncspiderImageInterface::SOURCE_URL => $syncspiderImage->getData('tag')
                    ]
                ]);
            }
        }

        return $data;
    }

    /**
     * Decorate given media gallery entries with "syncspider_image" extension attribute
     *
     * Extension attribute is populated for entries having data inside the "apiextension_images"
     * table
     *
     * {@inheritDoc}
     */
    public function getMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        $syncspiderImages = $this->getSyncspiderImages($mediaGalleryEntries);

        if (! empty($syncspiderImages)) {
            $processorContext = $context->getProcessorContext();

            foreach ($mediaGalleryEntries as $entry) {
                if (array_key_exists($entry->getId(), $syncspiderImages)) {
                    if ($processorContext->validEntity($entry)) {
                        $extensionAttributes = $processorContext->getEntityExtensionAttributes($entry);

                        if ($extensionAttributes instanceof ProductAttributeMediaGalleryEntryExtensionInterface) {
                            $extensionAttributes->setSyncspiderImage($syncspiderImages[$entry->getId()]);
                        }
                    }
                }
            }
        }

        return $mediaGalleryEntries;
    }
}