<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver;

use SyncSpider\APIextension\Model\ProductMediaGallery\{
    AreMediaGalleryEntriesEqualInteface,
    EntriesResolverContextInterface,
    EntriesResolverInterface
};

class RemoveDuplicates implements EntriesResolverInterface
{
    /**
     * @var AreMediaGalleryEntriesEqualInteface
     */
    private $areMediaGalleryEntriesEqual;

    public function __construct(AreMediaGalleryEntriesEqualInteface $areMediaGalleryEntriesEqual)
    {
        $this->areMediaGalleryEntriesEqual = $areMediaGalleryEntriesEqual;
    }

    /**
     * Move syncspider entries to the end of the gallery
     * 
     * Media entries are split into two lists magento source entries and
     * syncspider source entries. Magento source entries can contain duplicates
     * but syncspider source entries contain only unique images - any syncspider or 
     * magento source entries being duplicate on any unique syncspider source entry
     * are excluded from the list but moved to the syncspider source entry "syncspider_image"
     * extension attribute
     *
     * {@inheritDoc}
     */
    public function getMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        $magentoEntries = [];
        $syncspiderEntries = [];

        foreach ($mediaGalleryEntries as $entry) {
            $syncspiderImage = null;

            foreach ($syncspiderEntries as $syncspiderEntry) {
                if ($this->areMediaGalleryEntriesEqual->execute($entry, $syncspiderEntry)) {
                    $syncspiderImage = $syncspiderEntry;
                    break;
                }
            }

            if ($syncspiderImage) {
                // set entry as an duplicate to an syncspider source entry
                $syncspiderImage->getExtensionAttributes()
                    ->setMediaGalleryEntries(array_merge(
                        $syncspiderImage->getMediaGalleryEntries(),
                        $entry
                    ));
                continue;
            }

            $syncspiderImage = $context->getSyncspiderImage($entry);

            if ($syncspiderImage) {
                // set entry as unique syncspider source entry
                $syncspiderEntries[] = $entry;
            } else {
                // set not matched entry as magento source entry (can be duplicate)
                $magentoEntries[] = $entry;
            }
        }

        return array_merge(
            $magentoEntries,
            $syncspiderEntries
        );
    }
}