<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver;

use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolver\CleanEntriesTypes,
    AreMediaGalleryEntriesEqualInteface,
    EntriesResolverContextInterface,
    EntriesResolverInterface
};

class UniqueEntries implements EntriesResolverInterface
{
    /**
     * @var AreMediaGalleryEntriesEqualInteface
     */
    private $areMediaGalleryEntriesEqual;

    /**
     * @var CleanEntriesTypes
     */
    private $cleanEntriesTypes;

    public function __construct(
        AreMediaGalleryEntriesEqualInteface $areMediaGalleryEntriesEqual,
        CleanEntriesTypes $cleanEntriesTypes
    ) {
        $this->areMediaGalleryEntriesEqual = $areMediaGalleryEntriesEqual;
        $this->cleanEntriesTypes = $cleanEntriesTypes;
    }

    /**
     * Leave only unique images
     * 
     * Image sequence is preserved based on the first unique
     * image.
     *
     * {@inheritDoc}
     */
    public function getMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        $uniqueEntries = [];

        foreach ($mediaGalleryEntries as $entry) {
            $mainEntry = $entry;

            foreach ($uniqueEntries as $uniqueEntry) {
                if ($this->areMediaGalleryEntriesEqual->execute($entry, $uniqueEntry, $context)) {
                    $mainEntry = $uniqueEntry;
                    break;
                }
            }

            if ($mainEntry === $entry) {
                // unique entry processing
                if ($context->isCreateOperation() || $entry->getId()) {
                    $uniqueEntries[] = $entry;
                }
            } else {
                // inherit types and label
                $types = array_unique(
                    array_merge(
                        $mainEntry->getTypes() ?? [],
                        $entry->getTypes() ?? []
                    )
                );
                $mainEntry->setTypes($types)
                    ->setLabel(
                        $entry->getLabel() ?: $mainEntry->getLabel()
                    );
                $mainSyncspiderImage = $context->getSyncspiderImage($mainEntry);
                $syncspiderImage = $context->getSyncspiderImage($entry);

                // mark unique magento entry as syncspier image due it has an syncspider entry duplicate
                if (! $mainSyncspiderImage && $syncspiderImage) {
                    $extensionAttributes = $context->getProcessorContext()->getEntityExtensionAttributes($mainEntry);
                    $extensionAttributes->setSyncspiderImage($syncspiderImage);
                    $mainSyncspiderImage = $syncspiderImage;
                }

                // Upate unique syncspider image source url - prioritize latest syncspider image url
                if ($mainSyncspiderImage && $syncspiderImage && $syncspiderImage->getSourceUrl()) {
                    $mainSyncspiderImage->setSourceUrl($syncspiderImage->getSourceUrl());
                }

                // new entry has highest priority to the types in the stack
                if (! $entry->getId()) {
                    $this->cleanEntriesTypes->cleanTypes($uniqueEntries, $mainEntry);

                    // mark unique syncspider image as fresh due new entry matching existing unique syncspider image
                    if ($mainSyncspiderImage) {
                        $mainSyncspiderImage->setIsFresh(true);
                    }
                }
            }
        }

        return $uniqueEntries;
    }
}