<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery;

use Magento\Catalog\Api\Data\{
    ProductInterface,
    ProductAttributeMediaGalleryEntryInterface
};
use SyncSpider\APIextension\Api\Data\SyncspiderImageInterface;
use SyncSpider\APIextension\Api\ExtensionAttributes\ProcessorContextInterface;

interface EntriesResolverContextInterface
{
    public const OPERATION_CREATE = 'CREATE';

    public const OPERATION_UPDATE = 'UPDATE';

    public const FORCE_CLEAN_GALLERY = 'force_clean_gallery';

    public const PRESERVE_PREVIOUS_IMAGES = 'preserve_previous_images';

    /**
     * Get entries resolver context product
     *
     * @return ProductInterface
     */
    public function getProduct(): ProductInterface;

    /**
     * Get context operation type
     *
     * @return bool
     */
    public function getOperationType(): string;

    /**
     * Whether current context operation is create
     *
     * @return boolean
     */
    public function isCreateOperation(): bool;

    /**
     * Whether current context operation is update
     *
     * @return boolean
     */
    public function isUpdateOperation(): bool;

    /**
     * Get image entries for the entries resolver
     *
     * @return array
     */
    public function getImageEntries(): array;

    /**
     * Get entries resolver processor context
     *
     * @return ProcessorContextInterface
     */
    public function getProcessorContext(): ProcessorContextInterface;

    /**
     * Whether force clean gallery should be done
     *
     * @return boolean
     */
    public function isForceCleanGallery(): bool;

    /**
     * Whether preserve previous images
     *
     * @return boolean
     */
    public function isPreservePreviousImages(): bool;

    /**
     * Get syncspider image for given media gallery entry
     *
     * @param ProductAttributeMediaGalleryEntryInterface $entry
     *
     * @return SyncspiderImageInterface|null
     */
    public function getSyncspiderImage(ProductAttributeMediaGalleryEntryInterface $entry): ?SyncspiderImageInterface;

    /**
     * Get media gallery entries map
     *
     * Given list of media gallery entries is converted
     * to map of media gallery entries where key is the
     * entry id or other value returned by the mapKeyFn.
     * The mapKeyFn can return null which means the entry
     * will be exclued from the map
     *
     * @param array $mediaGalleryEntries
     * @param callable $mapKeyFn
     *
     * @return ProductAttributeMediaGalleryEntryInterface[]
     */
    public function getMediaGalleryEntriesMap(array $mediaGalleryEntries, callable $mapKeyFn = null): array;
}