<?php

namespace SyncSpider\APIextension\Model;

use Magento\Catalog\Api\Data\ProductInterface;
use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolverInterface,
    EntriesResolverContextInterface
};

class ProductMediaGalleryProcessor
{
    /**
     * @var EntriesResolverInterface[]
     */
    private $entriesResolverPool;

    public function __construct(
        array $entriesResolverPool = []
    ) {
        $this->entriesResolverPool = $entriesResolverPool;
    }

    /**
     * Apply stack of entries resolvers to product gallery entries
     *
     * @param ProductInterface $product
     * @param EntriesResolverContextInterface $context
     *
     * @return array
     */
    public function execute(ProductInterface $product, EntriesResolverContextInterface $context): array
    {
        $mediaGalleryEntries = $context->isForceCleanGallery() ? [] : $product->getMediaGalleryEntries();

        foreach ($this->entriesResolverPool as $entriesResolver) {
            $mediaGalleryEntries = $entriesResolver->getMediaGalleryEntries($mediaGalleryEntries, $context);
        }


        return $mediaGalleryEntries;
    }
}