<?php

namespace SyncSpider\APIextension\Model\ProductOption\InformationResolver;

use Magento\ConfigurableProduct\Api\Data\ConfigurableItemOptionValueInterface;
use SyncSpider\APIextension\Api\Data\SimpleOptionInterface;
use SyncSpider\APIextension\Model\ProductOption\{
    InformationResolverInterface,
    ProductOptionInformation,
    ProductOptionInformationFactory
};
use Magento\Framework\DataObject;

class ConfigurableItemOptionInformationResolver implements InformationResolverInterface
{
    /**
     * @var ProductOptionInformationFactory
     */
    private $informationFactory;

    public function __construct(ProductOptionInformationFactory $informationFactory)
    {
        $this->informationFactory = $informationFactory;
    }

    /**
     * @inheritDoc
     */
    public function validEntity($entity): bool
    {
        return $entity instanceof ConfigurableItemOptionValueInterface;
    }

    /**
     * @inheritDoc
     */
    public function entityInformationKeyFields($entity): array
    {
        if (! $this->validEntity($entity)) {
            return [];
        }

        return [
            ConfigurableItemOptionValueInterface::OPTION_ID => $entity->getOptionId(),
            ConfigurableItemOptionValueInterface::OPTION_VALUE => $entity->getOptionValue()
        ];
    }

    /**
     * @inheritDoc
     */
    public function getProductOptionInformation(DataObject $request): ProductOptionInformation
    {
        if (! $request->hasData(static::class)) {
            $optionInformation = $request->getData('product_options/attributes_info');
            $optionInformation = $this->informationFactory->create([
                'optionInformation' => is_array($optionInformation) ? $optionInformation : [],
                'requiredFields' => [
                    ConfigurableItemOptionValueInterface::OPTION_ID,
                    ConfigurableItemOptionValueInterface::OPTION_VALUE,
                    SimpleOptionInterface::LABEL,
                    SimpleOptionInterface::VALUE
                ],
                'keyFields' => [
                    ConfigurableItemOptionValueInterface::OPTION_ID,
                    ConfigurableItemOptionValueInterface::OPTION_VALUE
                ]
            ]);
            $request->setData(static::class, $optionInformation);
        }

        return $request->getData(static::class);
    }
}