<?php

namespace SyncSpider\APIextension\Model\ProductOption;

use Magento\Framework\DataObject;

class ProductOptionInformation
{
    const REQUEST_CONTEXT = 'product_options_buy_request';

    /**
     * @var string
     */
    private $optionInformation;

    /**
     * @var array
     */
    private $requiredFields;

    /**
     * @var array
     */
    private $keyFields;

    /**
     * @var array
     */
    private $informationMap;

    public function __construct(
        array $optionInformation,
        array $requiredFields,
        array $keyFields
    ) {
        $this->optionInformation = $optionInformation;
        $this->requiredFields = $requiredFields;
        $this->keyFields = $keyFields;
    }

    /**
     * Get information option path
     *
     * @return string
     */
    public function getOptionInformation(): string
    {
        return $this->optionInformation;
    }

    /**
     * Get information required fields
     *
     * @return array
     */
    public function getRequiredFields(): array
    {
        return $this->requiredFields;
    }

    /**
     * Get information key fields
     *
     * @return array
     */
    public function getKeyFields(): array
    {
        return $this->keyFields;
    }

    /**
     * Get information key for given fields
     *
     * @param array $fields
     *
     * @return string
     */
    public function getInformationKey(array $fields): string
    {
        $fields = array_intersect_key($fields, array_flip($this->getKeyFields()));
        return implode('_', $fields);
    }

    /**
     * Get attributes information map
     *
     * @param DataObject $request
     * @param string $optionPath
     * @param array $requiredFields
     * @param array $keyFields
     *
     * @return array
     */
    public function getInformationMap(): array
    {
        if (! isset($this->informationMap)) {
            $informationMap = [];
            $optionInformation = $this->optionInformation;
            $requiredInformation = array_fill_keys($this->getRequiredFields(), true);
    
            if (is_array($optionInformation)) {
                foreach ($optionInformation as $information) {
                    $information = array_intersect_key($information, $requiredInformation);
                    $information = array_filter($information, function ($value) {
                        return is_scalar($value);
                    });
    
                    if (count($requiredInformation) !== count($information)) {
                        continue;
                    }
    
                    $informationKey = $this->getInformationKey($information);
                    $informationMap[$informationKey] = $information;
                }
            }

            $this->informationMap = $informationMap;
        }

        return $this->informationMap;
    }
}