<?php

namespace SyncSpider\APIextension\Model;

use SyncSpider\APIextension\Api\RemoteResourceInterface;
use InvalidArgumentException;
use Exception;

class RemoteResource implements RemoteResourceInterface
{
    /**
     * @var resource
     */
    private $resource;

    /**
     * @var resource
     */
    private $stream;

    public function __construct($resource)
    {
        $this->resource = $resource;
        $this->stream = tmpfile();
        $this->guardFreshResource();
    }

    public function __destruct()
    {
        try {
            is_resource($this->stream) && fclose($this->stream);
            is_resource($this->resource) && fclose($this->resource);
        } catch (Exception $e) {
            // suppress odd resource closing exceptions which should never happen
        }
    }

    /**
     * @inheritDoc
     */
    public function guardFreshResource()
    {
        if (! is_resource($this->resource)
            || ftell($this->resource) !== 0
        ) {
            throw new InvalidArgumentException('Image resource must be valid resource with start position');
        }
    }

    /**
     * @inheritDoc
     */
    public function getMimeType(): string
    {
        if (ftell($this->stream) === 0
            && is_resource($this->resource)
            && ! feof($this->resource)
        ) {
            // read first 2000 bytes to determinate the mime type
            $header = stream_get_contents($this->resource, 2000);
            fwrite($this->stream, $header);
        }

        return mime_content_type($this->stream);
    }

    /**
     * @inheritDoc
     */
    public function getContents(): string
    {
        if (is_resource($this->resource) && ! feof($this->resource)) {
            // fetch all content and close the resource as it is no longer needed
            fwrite($this->stream, stream_get_contents($this->resource));
            fclose($this->resource);
        }

        $contents = stream_get_contents($this->stream, null, 0);
        return $contents ?? '';
    }
}