<?php

namespace SyncSpider\APIextension\Model\Rest\Swagger;

use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Webapi\Model\Rest\Swagger\Generator;

class ExtendedGenerator extends Generator
{
    /**
     * Attach writable fields to generated webapi data
     *
     * @param string $typeName
     *
     * @return array
     */
    protected function generateDefinition($typeName)
    {
        $definition = parent::generateDefinition($typeName);
        $typeData = $this->typeProcessor->getTypeData($typeName);
        $definition['writableFields'] = $typeData['writableFields'] ?? [];
        return $definition;
    }

    /**
     * Convert writable fields from camel case to snake case
     *
     * @return array
     */
    protected function getDefinitions()
    {
        $definitions = parent::getDefinitions();

        foreach ($definitions as $name => $definition) {
            if (! empty($definition['writableFields'])) {
                $writableFields = [];

                foreach ($definition['writableFields'] as $field) {
                    $writableFields[] = SimpleDataObjectConverter::camelCaseToSnakeCase($field);
                }

                $definitions[$name]['writableFields'] = $writableFields;
            }
        }

        return $definitions;
    }
}
