<?php

namespace SyncSpider\APIextension\Model;

use Magento\Framework\DataObject;
use SyncSpider\APIextension\Api\WebapiServiceInputInterface;
use Exception;

class WebapiServiceInput implements WebapiServiceInputInterface
{
    /**
     * Whether given value is instance of an data object
     *
     * @param mixed $value
     *
     * @return bool
     */
    public function isDataObject($value): bool
    {
        return $value instanceof DataObject;
    }

    /**
     * @inheritDoc
     */
    public function add($inputValue, $value): bool
    {
        return !! ($this->isDataObject($value) ? $value->setData(static::DATA_KEY, $inputValue) : false);
    }

    /**
     * @inheritDoc
     */
    public function has($value): bool
    {
        return $this->isDataObject($value) && $value->hasData(static::DATA_KEY);
    }

    /**
     * @inheritDoc
     */
    public function get($value)
    {
        if (! $this->has($value)) {
            throw new Exception('No webapi service input value for given value exists');
        }

        return $value->getData(static::DATA_KEY);
    }

    /**
     * @inheritDoc
     */
    public function remove($value)
    {
        if ($this->has($value)) {
            $value->unsetData($value);
            return $value;
        }

        if (is_array($value) && array_key_exists(static::DATA_KEY, $value)) {
            unset($value[static::DATA_KEY]);
            return $value;
        }

        return $value;
    }
}
