<?php

namespace SyncSpider\APIextension\Observer\EntityManager;

use Magento\Framework\{
    Event\Observer,
    Event\ObserverInterface
};
use Magento\Framework\Data\Collection\AbstractDb;
use SyncSpider\APIextension\EntityManager\ExtensionAttributesResolver;
use Exception;

class LegacyCollectionEventObserver implements ObserverInterface
{
    /**
     * @var ExtensionAttributesResolver
     */
    private $extensionAttributesResolver;

    /**
     * @var string
     */
    private $eventFlag;

    /**
     * @var string
     */
    private $actionName;

    public function __construct(
        ExtensionAttributesResolver $extensionAttributesResolver,
        string $eventFlag = 'core_collection_abstract_load_after',
        string $actionName = null
    ) {
        $this->extensionAttributesResolver = $extensionAttributesResolver;
        $this->eventFlag = $eventFlag;
        $this->actionName = $actionName;
    }

    /**
     * Trigger entity manager "collection" extension actions on modules that do not support them out of the box
     *
     * @param Observer $observer
     */
    public function execute(Observer $observer)
    {
        $collection = $observer->getEvent()->getData('collection');

        if ($collection instanceof AbstractDb
            && ! $this->extensionAttributesResolver->hasActionFlag($collection, $this->eventFlag)
        ) {
            $this->extensionAttributesResolver->setActionFlag($collection, $this->eventFlag);

            try {
                $entity = $collection->getNewEmptyItem();
                $entityTypes = [];

                if ($this->extensionAttributesResolver->hasExtensibleInterface($entity)) {
                    $entityTypes[] = $this->extensionAttributesResolver->getExtensibleInterfaceName($entity);
                }

                foreach ($entityTypes as $entityType) {
                    $actionName = $this->actionName;
                    $this->extensionAttributesResolver->triggerEntityActions($collection, $entityType, $actionName);
                }
            } catch (Exception $e) {
                // suppress exceptions
            }
        }
    }
}
