<?php

namespace SyncSpider\APIextension\Observer\EntityManager;

use Magento\Framework\{
    Event\Observer,
    Event\ObserverInterface
};
use SyncSpider\APIextension\EntityManager\ExtensionAttributesResolver;
use Exception;

class LegacyEntityEventsObserver implements ObserverInterface
{
    /**
     * @var ExtensionAttributesResolver
     */
    private $extensionAttributesResolver;
    /**
     * @var string
     */
    private $actionName;

    public function __construct(
        ExtensionAttributesResolver $extensionAttributesResolver,
        string $actionName = null
    ) {
        $this->extensionAttributesResolver = $extensionAttributesResolver;
        $this->actionName = $actionName;
    }

    /**
     * Trigger entity manager extension action on modules that do not support them out of the box
     *
     * @param Observer $observer
     *
     * @return void
     */
    public function execute(Observer $observer)
    {
        try {
            $entity = $observer->getEvent()->getData('object');

            if (! $this->extensionAttributesResolver->hasActionFlag($entity, ExtensionAttributesResolver::IS_ENTITY_MANAGER_ENTITY)) {
                $entityTypes = [];

                if ($this->extensionAttributesResolver->hasExtensibleInterface($entity)) {
                    $entityTypes[] = $this->extensionAttributesResolver->getExtensibleInterfaceName($entity);
                }

                foreach ($entityTypes as $entityType) {
                    $actionName = $this->actionName;
                    $this->extensionAttributesResolver->triggerEntityActions($entity, $entityType, $actionName);
                }
            }
        } catch (Exception $e) {
            // suppress exceptions
        }
    }
}
