<?php

namespace SyncSpider\APIextension\Observer\EntityManager;

use Magento\Framework\{
    Event\Observer,
    Event\ObserverInterface,
    Model\AbstractModel
};
use SyncSpider\APIextension\EntityManager\ExtensionAttributesResolver;
use Exception;

class LegacyEntityIsObjectNewObserver implements ObserverInterface
{
    /**
     * @var ExtensionAttributesResolver
     */
    private $extensionAttributesResolver;
    /**
     * @var string
     */
    private $actionName;

    public function __construct(
        ExtensionAttributesResolver $extensionAttributesResolver,
        string $actionName = null
    ) {
        $this->extensionAttributesResolver = $extensionAttributesResolver;
        $this->actionName = $actionName;
    }

    /**
     * Toggle "is_object_new" so entity manager extension action can trigger update action
     *
     * @param Observer $observer
     *
     * @return void
     */
    public function execute(Observer $observer)
    {
        try {
            $entity = $observer->getEvent()->getData('object');

            if ($entity && $entity instanceof AbstractModel) {
                switch ($observer->getEvent()->getName()) {
                    case 'model_save_before':
                        if ($entity->isObjectNew()) {
                            $this->extensionAttributesResolver->setActionFlag($entity, ExtensionAttributesResolver::IS_OBJECT_NEW_FLAG);
                        }
                        break;
                    case 'model_save_after':
                        $this->extensionAttributesResolver->removeActionFlag($entity, ExtensionAttributesResolver::IS_OBJECT_NEW_FLAG);
                        break;
                }    
            }
        } catch (Exception $e) {
            // suppress exceptions
        }
    }
}
