<?php

namespace SyncSpider\APIextension\Plugin\Magento\Catalog\Api;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Api\Data\CategoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Catalog\Api\CategoryListInterface;

class CategoryRepositoryInterfacePlugin
{
    public const HANDLE_CATEGORY_URL_COLLUSION = 'syncspider/general/handle_category_url_collusion';

    /**
     * @var CategoryListInterface
     */
    private $categoryList;

    /**
     * @var SearchCriteriaBuilderFactory
     */
    private $searchCriteriaBuilderFactory;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * ProductRepositoryInterfacePlugin constructor.
     *
     * @param CategoryListInterface $categoryList
     * @param SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        CategoryListInterface $categoryList,
        SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->categoryList = $categoryList;
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * Whether the category url colussion should be handled
     *
     * @return bool
     */
    public function handleCategoryUrlCollusion()
    {
        return !! $this->scopeConfig->getValue(static::HANDLE_CATEGORY_URL_COLLUSION);
    }

    /**
     * Get existing category matching given category entity name and parent
     *
     * @param CategoryInterface $sourceCategory
     *
     * @return CategoryInterface|null
     */
    public function existingCategory(CategoryInterface $sourceCategory): ?CategoryInterface
    {
        $searchCriteriaBuilder = $this->searchCriteriaBuilderFactory->create()
            ->addFilter('parent_id', $sourceCategory->getParentId())
            ->addFilter('name', $sourceCategory->getName())
            ->setPageSize(1);
        $categories = $this->categoryList->getList($searchCriteriaBuilder->create());

        foreach ($categories->getItems() as $item) {
            return $item;
        }

        return null;
    }

    /**
     * Set category entity id to already existing category matching category name and parent id
     *
     * Setting category id of existing category to cause category update instead of creation as creation
     * would generate an "url key already exists" error
     *
     * @param CategoryRepositoryInterface $subject
     * @param CategoryInterface $category
     */
    public function beforeSave(CategoryRepositoryInterface $subject, CategoryInterface $category)
    {
        if ($this->handleCategoryUrlCollusion()
            && ! $category->getId()
            && $category->getName()
            && $category->getParentId()
        ) {
            $existingCategory = $this->existingCategory($category);

            if ($existingCategory) {
                $category->setId($existingCategory->getId());
            }
        }
    }
}
