<?php

namespace SyncSpider\APIextension\Plugin\Magento\Catalog\Model;

use Magento\Catalog\Api\Data\CustomOptionInterface;
use Magento\Catalog\Model\ProductOptionProcessor;
use Magento\Framework\DataObject;
use SyncSpider\APIextension\Api\ExtensionAttributes\{
    ProcessorInterface,
    ProcessorContextInterfaceFactory
};
use SyncSpider\APIextension\Model\ProductOption\ProductOptionInformation;

class ProductOptionProcessorPlugin
{
    /**
     * @var ProcessorInterface
     */
    private $processor;

    /**
     * @var ProcessorContextInterfaceFactory
     */
    private $contextFactory;

    public function __construct(
        ProcessorInterface $processor,
        ProcessorContextInterfaceFactory $contextFactory
    ) {
        $this->processor = $processor;
        $this->contextFactory = $contextFactory;
    }

    /**
     * Get configurable item options from given data
     *
     * @param array $data
     *
     * @return CustomOptionInterface[]
     */
    public function getCustomOptions($data)
    {
        $configurableItemOptions = [];
    
        if (isset($data['custom_options'])
            && is_array($data['custom_options'])
        ) {
            $configurableItemOptions = $data['custom_options'];
            $configurableItemOptions = array_filter($configurableItemOptions, function ($value) {
                return $value instanceof CustomOptionInterface;
            });
        }

        return $configurableItemOptions;
    }

    /**
     * Attach additional extension attributes to custom option values
     *
     * @param ProductOptionProcessor $subject
     * @param array $result
     * @param DataObject $request
     *
     * @return array
     */
    public function afterConvertToProductOption(ProductOptionProcessor $subject, $result, DataObject $request)
    {
        $customOptions = $this->getCustomOptions($result);
        $context = $this->contextFactory->create([
            'context' => [
                ProductOptionInformation::REQUEST_CONTEXT => $request
            ]
        ]);
        
        foreach ($customOptions as $item) {
            $this->processor->process($item, $context);
        }

        return $result;
    }
}