<?php

namespace SyncSpider\APIextension\Plugin\Magento\ConfigurableProduct\Model;

use Magento\Framework\DataObject;
use Magento\ConfigurableProduct\Api\Data\ConfigurableItemOptionValueInterface;
use Magento\ConfigurableProduct\Model\ProductOptionProcessor;
use SyncSpider\APIextension\Api\ExtensionAttributes\{
    ProcessorInterface,
    ProcessorContextInterfaceFactory
};
use SyncSpider\APIextension\Model\ProductOption\ProductOptionInformation;

class ProductOptionProcessorPlugin
{
    /**
     * @var ProcessorInterface
     */
    private $processor;

    /**
     * @var ProcessorContextInterfaceFactory
     */
    private $contextFactory;

    public function __construct(
        ProcessorInterface $processor,
        ProcessorContextInterfaceFactory $contextFactory
    ) {
        $this->processor = $processor;
        $this->contextFactory = $contextFactory;
    }

    /**
     * Get configurable item options from given data
     *
     * @param array $data
     *
     * @return ConfigurableItemOptionValueInterface[]
     */
    public function getConfigurableItemOptions($data)
    {
        $configurableItemOptions = [];
    
        if (isset($data['configurable_item_options'])
            && is_array($data['configurable_item_options'])
        ) {
            $configurableItemOptions = $data['configurable_item_options'];
            $configurableItemOptions = array_filter($configurableItemOptions, function ($value) {
                return $value instanceof ConfigurableItemOptionValueInterface;
            });
        }

        return $configurableItemOptions;
    }

    /**
     * Attach additional extension attributes to configurable item option values
     *
     * @param ProductOptionProcessor $subject
     * @param array $result
     * @param DataObject $request
     *
     * @return array
     */
    public function afterConvertToProductOption(ProductOptionProcessor $subject, $result, DataObject $request)
    {
        $configurableItemOptions = $this->getConfigurableItemOptions($result);
        $context = $this->contextFactory->create([
            'context' => [
                ProductOptionInformation::REQUEST_CONTEXT => $request
            ]
        ]);

        foreach ($configurableItemOptions as $item) {
            $this->processor->process($item, $context);
        }

        return $result;
    }
}