<?php

namespace SyncSpider\APIextension\Plugin\Magento\Customer\Model;

use Magento\Customer\Api\Data\AddressInterface;
use Magento\Customer\Api\Data\RegionInterfaceFactory;
use Magento\Customer\Model\Address;
use Exception;

class AddressPlugin
{
    /**
     * @var RegionInterfaceFactory
     */
    private $regionFactory;

    /**
     * AddressPlugin constructor.
     *
     * @param RegionInterfaceFactory $regionFactory
     */
    public function __construct(RegionInterfaceFactory $regionFactory)
    {
        $this->regionFactory = $regionFactory;
    }

    /**
     * Populate source address region object if it is missing
     *
     * @param Address $subject
     * @param AddressInterface $address
     */
    public function beforeUpdateData(Address $subject, AddressInterface $address)
    {
        if (! $address->getRegion()) {
            $region = $this->regionFactory->create();

            if ($address->getRegionId()) {
                try {
                    $regionModel = $subject->getRegionModel($address->getRegionId());
                    $region->setRegion($regionModel->getName())
                        ->setRegionId($regionModel->getRegionId())
                        ->setRegionCode($regionModel->getCode());
                } catch (Exception $e) {
                    // suppress exceptions
                }

            }

            $address->setRegion($region);
        }
    }
}
