<?php

namespace SyncSpider\APIextension\Plugin\Magento\Framework;

use Magento\Framework\DataObject;
use SyncSpider\APIextension\Api\WebapiServiceInputInterface;

class DataObjectPlugin
{
    /**
     * @var WebapiServiceInputInterface
     */
    private $webapiServiceInput;

    public function __construct(WebapiServiceInputInterface $webapiServiceInput)
    {
        $this->webapiServiceInput = $webapiServiceInput;
    }

    /**
     * Remove webapi service input value when reading data object
     *
     * @param DataObject $subject
     * @param array $result
     * @param string $key
     * @param string|int $index
     *
     * @return array
     */
    public function afterGetData(DataObject $subject, $result, $key = '', $index = null)
    {
        if ($this->webapiServiceInput->has($subject) && empty($key) && is_array($result)) {
            $result = $this->webapiServiceInput->remove($result);
        }

        return $result;
    }

    /**
     * Remove webapi service input value when converting data object to array
     *
     * @param DataObject $subject
     * @param array $result
     *
     * @return array
     */
    public function toArray(DataObject $subject, $result)
    {
        if ($this->webapiServiceInput->has($subject)) {
            $result = $this->webapiServiceInput->remove($result);
        }

        return $result;
    }
}