<?php

namespace SyncSpider\APIextension\Plugin\Magento\Framework\EntityManager\Operation;

use Magento\Framework\EntityManager\Operation\DeleteInterface;
use SyncSpider\APIextension\EntityManager\ExtensionAttributesResolver;

class DeleteInterfacePlugin
{
    /**
     * @var ExtensionAttributesResolver
     */
    private $extensionAttributesResolver;

    public function __construct(ExtensionAttributesResolver $extensionAttributesResolver)
    {
        $this->extensionAttributesResolver = $extensionAttributesResolver;
    }

    /**
     * Set "IS_ENTITY_MANAGER_ENTITY" flag on entity
     *
     * Added to prevent multiple executions of entity actions, for example,
     * when product is saved it will trigger entity manager actions then legacy events
     * using the Magento\Framework\EntityManager\Observer\* if they are used which will
     * trigger additional entity actions execution due observer in extension which 
     *
     * @param DeleteInterface $subject
     * @param callable $proceed
     * @param object $entity
     * @param array $arguments
     *
     * @return object
     */
    public function aroundExecute(DeleteInterface $subject, callable $proceed, $entity, $arguments = [])
    {
        $flagOwner = false;

        if (! $this->extensionAttributesResolver->hasActionFlag($entity, ExtensionAttributesResolver::IS_ENTITY_MANAGER_ENTITY)) {
            $flagOwner = $this->extensionAttributesResolver->setActionFlag($entity, ExtensionAttributesResolver::IS_ENTITY_MANAGER_ENTITY);
        }

        $result = $proceed($entity, $arguments);

        if ($flagOwner) {
            $this->extensionAttributesResolver->removeActionFlag($entity, ExtensionAttributesResolver::IS_ENTITY_MANAGER_ENTITY);
        }

        return $result;
    }
}
