<?php

namespace SyncSpider\APIextension\Plugin\Magento\Framework\Reflection;

use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Framework\Reflection\TypeProcessor;
use Magento\Framework\Reflection\NameFinder;
use Zend\Code\Reflection\ClassReflection;
use ReflectionMethod;
use LogicException;
use Exception;

class TypeProcessorPlugin
{
    /**
     * @var NameFinder
     */
    private $nameFinder;

    /**
     * @var bool[]
     */
    private $writableCache;

    /**
     * TypeProcessorPlugin constructor
     *
     * @param NameFinder $nameFinder
     */
    public function __construct(NameFinder $nameFinder)
    {
        $this->nameFinder = $nameFinder;
    }

    /**
     * Attach writable fields information to type data
     *
     * @param TypeProcessor $subject
     * @param TypeProcessor $result
     * @param string $type
     *
     * @return TypeProcessor
     */
    public function afterRegister(TypeProcessor $subject, $result, $type)
    {
        $typeName = $result !== $type ? $result : null;

        if ($typeName && ! $subject->isArrayType($typeName) && ! isset($this->writableCache[$typeName])) {
            try {
                $typeData = $subject->getTypeData($typeName);
                $this->writableCache[$typeName] = true;
                $parameters = $typeData['parameters'] ?? [];
                $writableFields = [];
                $class = new ClassReflection($type);

                foreach ($class->getMethods(ReflectionMethod::IS_PUBLIC) as $method) {
                    $fieldName = $this->nameFinder->getFieldNameFromGetterName($method->getName());

                    // skip methods which are not present is type parameters as they are not "getter" methods
                    if (! isset($parameters[$fieldName])) {
                        continue;
                    }

                    $writableFields[$fieldName] = ['writable' => false];

                    try {
                        $setterName = $this->nameFinder->getSetterMethodName(
                            $method->getDeclaringClass(),
                            SimpleDataObjectConverter::snakeCaseToUpperCamelCase($fieldName)
                        );
                        $writableFields[$fieldName]['writable'] = true;
                    } catch (LogicException $e) {
                        // suppress exceptions in case no setter exist for field
                    }
                }

                // extend type data with writable fields
                if (! empty($writableFields)) {
                    $subject->setTypeData($typeName, [
                        'parameters' => $writableFields,
                        'writableFields' => array_keys($writableFields, ['writable' => true], true)
                    ]);
                }
            } catch (Exception $e) {
                // suppress all exceptions so the default magento flow is not altered and exclude the type from processing
                $this->writableCache[$typeName] = false;
            }
        }

        return $result;
    }
}
