<?php

namespace SyncSpider\APIextension\Plugin\Magento\Framework\Setup\Declaration\Schema\Declaration\ValidationRules;

use Magento\Framework\Setup\Declaration\Schema\Declaration\ValidationRules\IncosistentReferenceDefinition;

class IncosistentReferenceDefinitionPlugin
{
    /**
     * Whether column error with error message is skipable
     *
     * @param string|null $column
     * @param string|null $message
     *
     * @return boolean
     */
    public function isSkipable(string $column = null, string $message = null)
    {
        $triggerColumn = 'image_id';
        $triggerMessage = 'Column definition "%s" and reference column definition "%s" are different in tables "%s" and "%s"';
        $triggerMessage = sprintf(
            $triggerMessage,
            $triggerColumn,
            'value_id',
            'apiextension_images',
            'catalog_product_entity_media_gallery'
        );
        return $column === $triggerColumn && $message === $triggerMessage;
    }

    /**
     * Silence unnecessary declarative shcema validation errors
     *
     * Needed to remove "padding" error on "catalog_product_entity_media_gallery" table "value_id" column as
     * on magento 2.3.x due it defines padding but on magento 2.4.x the padding is removed
     *
     * @param IncosistentReferenceDefinition $subject
     * @param array $result
     *
     * @return void
     */
    public function afterValidate(IncosistentReferenceDefinition $subject, array $result)
    {
        if (! empty($result)) {
            $errors = [];

            foreach ($result as $i => $error) {
                $column = $error['column'] ?? null;
                $message = $error['message'] ?? null;

                // remove skipable errors
                if ($this->isSkipable($column, $message)) {
                    continue;
                }

                $errors[] = $error;
            }

            $result = $errors;
        }

        return $result;
    }
}
