<?php

namespace SyncSpider\APIextension\Plugin\Magento\Framework\Setup\Declaration\Schema;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Setup\Declaration\Schema\{
    Diff\DiffInterface,
    DryRunLogger,
    OperationsExecutor
};
use Magento\Framework\Setup\Patch\{
    PatchApplier,
    PatchHistory
};

class OperationsExecutorPlugin
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var PatchApplier
     */
    private $patchApplier;

    public function __construct(ResourceConnection $resourceConnection, PatchApplier $patchApplier)
    {
        $this->resourceConnection = $resourceConnection;
        $this->patchApplier = $patchApplier;
    }

    /**
     * Apply syncspider custom data patches and then custom schema patches before declarative schema
     *
     * @param OperationsExecutor $subject
     * @param DiffInterface $diff
     * @param array $requestData
     *
     * @return void
     */
    public function beforeExecute(OperationsExecutor $subject, DiffInterface $diff, array $requestData)
    {
        $dryRun = isset($requestData[DryRunLogger::INPUT_KEY_DRY_RUN_MODE])
            && $requestData[DryRunLogger::INPUT_KEY_DRY_RUN_MODE];
        $patchTable = $this->resourceConnection->getTableName(PatchHistory::TABLE_NAME);

        if (! $dryRun && $this->resourceConnection->getConnection()->isTableExists($patchTable)) {
            $this->patchApplier->applyDataPatch('SyncSpider_APIextension');
            $this->patchApplier->applySchemaPatch('SyncSpider_APIextension');
        }
    }
}