<?php

namespace SyncSpider\APIextension\Plugin\Magento\InventoryConfigurationApi\Api\Data;

use Magento\InventoryConfigurationApi\Api\Data\StockItemConfigurationInterface;
use SyncSpider\APIextension\Api\InventoryStockServiceInterface;

class StockItemConfigurationInterfacePlugin
{
    /**
     * @var InventoryStockServiceInterface
     */
    private $stockService;

    public function __construct(InventoryStockServiceInterface $stockService)
    {
        $this->stockService = $stockService;
    }

    /**
     * Enable "backorders" funcionality based on custom inventory stock "backorders" feature
     *
     * Used to force "backorders" funcionality to allow importing orders having out of stock products
     *
     * @param StockItemConfigurationInterface $subject
     * @param int $result
     *
     * @return int
     */
    public function afterGetBackorders(StockItemConfigurationInterface $subject, $result)
    {
        $backorders = $this->stockService->backorders();

        if ($backorders->active()) {
            return StockItemConfigurationInterface::BACKORDERS_YES_NONOTIFY;
        }

        return $result;
    }

    /**
     * Disable use config manager stock funcionality based on custom inventory stock "backorders" feature
     *
     * Used to allow overwritting manage stock functionality to allow importing orders having out of stock products
     *
     * @param StockItemConfigurationInterface $subject
     * @param bool $result
     *
     * @return bool
     */
    public function afterIsUseConfigManageStock(StockItemConfigurationInterface $subject, $result)
    {
        $backorders = $this->stockService->backorders();

        if ($backorders->active()) {
            return false;
        }

        return $result;
    }

    /**
     * Disable stock item managment funcionality based on custom inventory stock "backorders" feature
     *
     * Used to allow disabling inventory stock managment functionality to allow importing
     * orders having out of stock products
     *
     * @return bool
     */
    public function afterIsManageStock(StockItemConfigurationInterface $subject, $result)
    {
        $backorders = $this->stockService->backorders();

        if ($backorders->active()) {
            return false;
        }

        return $result;
    }
}
