<?php

namespace SyncSpider\APIextension\Plugin\Magento\Sales\Api\Data;

use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Catalog\Api\{
    Data\ProductOptionInterface,
    Data\ProductOptionInterfaceFactory,
    Data\ProductOptionExtensionInterfaceFactory
};
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Sales\Model\Order\Item;
use SyncSpider\APIextension\Model\{
    ObjectManager\FactoryProcessor,
    ProductOption\ProductOptionProcessor
};

class OrderItemPlugin
{
    /**
     * @var $productOptionFactory
     */
    private $productOptionFactory;

    /**
     * @var ProductOptionExtensionInterfaceFactory
     */
    private $productOptionExtensionFactory;

    /**
     * @var FactoryProcessor
     */
    private $factoryProcessor;

    /**
     * @var ProductOptionProcessor
     */
    private $productOptionProcessor;

    public function __construct(
        ProductOptionInterfaceFactory $productOptionFactory,
        ProductOptionExtensionInterfaceFactory $productOptionExtensionFactory,
        FactoryProcessor $factoryProcessor,
        ProductOptionProcessor $productOptionProcessor
    ) {
        $this->productOptionFactory = $productOptionFactory;
        $this->productOptionExtensionFactory = $productOptionExtensionFactory;
        $this->factoryProcessor = $factoryProcessor;
        $this->productOptionProcessor = $productOptionProcessor;
        // Process factory classes
        $this->factoryProcessor
            ->process($this->productOptionFactory)
            ->process($this->productOptionExtensionFactory);
    }

    /**
     * Add additional product option extension attributes
     *
     * @param OrderItemInterface $subject
     * @param \Magento\Catalog\Api\Data\ProductOptionInterface|null $result
     *
     * @return \Magento\Catalog\Api\Data\ProductOptionInterface|null
     */
    public function afterGetProductOption(OrderItemInterface $subject, $result)
    {
        if (! $result instanceof ProductOptionInterface) {
            $result = $this->productOptionFactory->create();
            $subject->setProductOption($result);
        }

        if (! $result->getExtensionAttributes()) {
            $result->setExtensionAttributes($this->productOptionExtensionFactory->create());
        }

        if ($subject instanceof Item) {
            $request = $subject->getBuyRequest();
            $request->setProductOptions($subject->getProductOptions());
            $productOptions = $this->productOptionProcessor->convertToProductOption($request);
            $extensionAttributes = $result->getExtensionAttributes();

            if ($extensionAttributes) {
                foreach ($productOptions as $extensionAttribute => $value) {
                    $setter = SimpleDataObjectConverter::snakeCaseToCamelCase('set_' . $extensionAttribute);

                    if (method_exists($extensionAttributes, $setter)) {
                        $extensionAttributes->{$setter}($value);
                    }
                }
            }
        }

        return $result;
    }
}
