<?php

namespace SyncSpider\APIextension\Plugin\Magento\Sales\Model\Order;

use Magento\Framework\App\Config as AppConfig;
use SyncSpider\APIextension\Model\Order\Configuration;

class Config
{
    /**
     * @var Configuration
     */
    private $configuration;

    /**
     * @param Configuration $configuration
     */
    public function __construct(Configuration $configuration)
    {
        $this->configuration = $configuration;
    }

    /**
     * Implement plugin to allow overwriting configuration option values with custom values
     *
     * @param AppConfig $subject
     * @param callable $callback
     * @param array $arguments
     *
     * @return mixed
     */
    public function aroundGetValue(AppConfig $subject, callable $callback, ...$arguments)
    {
        if (! $this->canExecute()) {
            return empty($arguments) ? $callback() : $callback(...$arguments);
        }

        $path = isset($arguments[0]) ? $arguments[0] : null;
        $scope = isset($arguments[1]) ? $arguments[1] : null;

        if (!is_string($path) || !is_string($scope)) {
            return $callback(...$arguments);
        }

        $overriddenValue = $this->configuration->getValue($path);

        if ($overriddenValue !== null) {
            return $overriddenValue;
        }

        return $callback(...$arguments);
    }

    /**
     * Whether usage of custom configuration is required
     *
     * @return bool
     */
    protected function canExecute()
    {
        return $this->configuration->hasChanges();
    }
}
