<?php

namespace SyncSpider\APIextension\Plugin\Magento\Sales\Model\Order;

use Magento\Sales\Model\Order\Item;
use Magento\Catalog\Api\Data\ProductOptionInterface;
use SyncSpider\APIextension\Model\ProductOption\ProductOptionProcessor;

class ItemPlugin
{
    /**
     * @var ProductOptionProcessor
     */
    private $productOptionProcessor;

    public function __construct(ProductOptionProcessor $productOptionProcessor)
    {
        $this->productOptionProcessor = $productOptionProcessor;
    }

    /**
     * Add additional options data (info_buyRequest, options) based on item product option
     *
     * @param Item $subject
     * @param array $options
     *
     * @return array
     */
    public function beforeSetProductOptions(Item $subject, $options)
    {
        $productOption = $subject->getProductOption();

        if ($productOption instanceof ProductOptionInterface) {
            $productOptions = [
                'info_buyRequest' => $this->productOptionProcessor->convertToBuyRequest($productOption)->getData(),
                'options' => $this->productOptionProcessor->convertToVisibleOptions($productOption)
            ];

            foreach ($productOptions as $path => $value) {
                if (! empty($value)) {
                    if (! is_array($options)) {
                        $options = [];
                    }

                    if (! isset($options[$path]) || ! is_array($options[$path])) {
                        $options[$path] = [];
                    } else {
                        $options[$path] = $this->productOptionProcessor->removeReadOnlyOptions($options[$path]);
                    }

                    $options[$path] = array_merge(
                        $options[$path],
                        $value
                    );
                }
            }
        }

        return [
            $options
        ];
    }
}
