<?php

namespace SyncSpider\APIextension\Plugin\Magento\Webapi\Model\Config;

use Magento\Webapi\Model\Config\ClassReflector;
use LogicException;
use InvalidArgumentException;

class ClassReflectorPlugin
{
    /**
     * Remove empty values from class method reflection
     *
     * Empty values are added due the logic and invalid argument exceptions when there are issues with the docblocks
     *
     * @param ClassReflector $subject
     * @param callable $proceed
     * @param string $className
     * @param string[]|array $methods
     *
     * @return array
     *
     * @see aroundExtractMethodData
     */
    public function aroundReflectClassMethods(ClassReflector $subject, callable $proceed, $className, $methods)
    {
        $result = $proceed($className, $methods);
        $result = array_filter($result, function ($value) {
            return ! empty($value);
        });
        return $result;
    }

    /**
     * Continue webservice reflection even if the method data generates error
     *
     * Handle the logic and invalid exception to continue webservice reflection
     *
     * @param ClassReflector $subject
     * @param callable $proceed
     * @param object $method
     *
     * @return array
     *
     * @see \Magento\Framework\Reflection\TypeProcessor::translateTypeName
     * @see \Magento\Framework\Reflection\TypeProcessor::register
     */
    public function aroundExtractMethodData(ClassReflector $subject, callable $proceed, $method)
    {
        try {
            $result = $proceed($method);
        } catch (LogicException | InvalidArgumentException $e) {
            $result = [
                'documentation' => 'Unextracted method data due exception: ' . $e->getMessage(),
                'interface' => []
            ];
        }

        return $result;
    }
}
